class BadImageFormatException(SystemException,ISerializable,_Exception):
 """
 The exception that is thrown when the file image of a dynamic link library (DLL) or an executable program is invalid.

 

 BadImageFormatException()

 BadImageFormatException(message: str)

 BadImageFormatException(message: str,inner: Exception)

 BadImageFormatException(message: str,fileName: str)

 BadImageFormatException(message: str,fileName: str,inner: Exception)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: BadImageFormatException,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo object with the file name,assembly 

    cache log,and additional exception information.

  

  

   info: The System.Runtime.Serialization.SerializationInfo that holds the serialized object data about 

    the exception being thrown.

  

   context: The System.Runtime.Serialization.StreamingContext that contains contextual information about the 

    source or destination.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def ToString(self):
  """
  ToString(self: BadImageFormatException) -> str

  

   Returns the fully qualified name of this exception and possibly the error message,the name of 

    the inner exception,and the stack trace.

  

   Returns: A string containing the fully qualified name of this exception and possibly the error message,

    the name of the inner exception,and the stack trace.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,message=None,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,inner: Exception)

  __new__(cls: type,message: str,fileName: str)

  __new__(cls: type,message: str,fileName: str,inner: Exception)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 FileName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the file that causes this exception.



Get: FileName(self: BadImageFormatException) -> str



"""

 FusionLog=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the log file that describes why an assembly load failed.



Get: FusionLog(self: BadImageFormatException) -> str



"""

 Message=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the error message and the name of the file that caused this exception.



Get: Message(self: BadImageFormatException) -> str



"""





"""Database elements deletion functions."""
from pyrevit import DOCS
from pyrevit.framework import List
from pyrevit import DB
from pyrevit.revit.db import query
from pyrevit.revit.db import ensure


def clear_sheet_revisions(sheet):
    sheet.SetAdditionalRevisionIds(List[DB.ElementId]([]))


def delete_elements(element_list, doc=None):
    doc = doc or DOCS.doc
    element_ids = ensure.ensure_element_ids(element_list)
    return doc.Delete(List[DB.ElementId](element_ids))


def delete_revision(rvt_rev, doc=None):
    doc = doc or DOCS.doc
    return doc.Delete(rvt_rev.Id)


def reset_subcategories(doc=None, purgable=False, filterfunc=None):
    # get subcategories
    cats_to_delete = query.get_subcategories(doc=doc,
                                             purgable=purgable,
                                             filterfunc=filterfunc)
    doc.Delete(List[DB.ElementId]([x.Id for x in cats_to_delete]))
    del cats_to_delete



"""Add x/html serialization to Elementree.

Taken from ElementTree 1.3 preview with slight modifications

Copyright (c) 1999-2007 by Fredrik Lundh.  All rights reserved.

fredrik@pythonware.com
http://www.pythonware.com

--------------------------------------------------------------------
The ElementTree toolkit is

Copyright (c) 1999-2007 by Fredrik Lundh

By obtaining, using, and/or copying this software and/or its
associated documentation, you agree that you have read, understood,
and will comply with the following terms and conditions:

Permission to use, copy, modify, and distribute this software and
its associated documentation for any purpose and without fee is
hereby granted, provided that the above copyright notice appears in
all copies, and that both that copyright notice and this permission
notice appear in supporting documentation, and that the name of
Secret Labs AB or the author not be used in advertising or publicity
pertaining to distribution of the software without specific, written
prior permission.

SECRET LABS AB AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH REGARD
TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANT-
ABILITY AND FITNESS.  IN NO EVENT SHALL SECRET LABS AB OR THE AUTHOR
BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY
DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
OF THIS SOFTWARE.
"""

from __future__ import absolute_import
from __future__ import unicode_literals

from . import util

ElementTree = util.etree.ElementTree
QName = util.etree.QName
if hasattr(util.etree, 'test_comment'):  # pragma: no cover
    Comment = util.etree.test_comment
else:  # pragma: no cover
    Comment = util.etree.Comment
PI = util.etree.PI
ProcessingInstruction = util.etree.ProcessingInstruction

__all__ = ['to_html_string', 'to_xhtml_string']

HTML_EMPTY = ("area", "base", "basefont", "br", "col", "frame", "hr",
              "img", "input", "isindex", "link", "meta" "param")

try:
    HTML_EMPTY = set(HTML_EMPTY)
except NameError:  # pragma: no cover
    pass

_namespace_map = {
    # "well-known" namespace prefixes
    "http://www.w3.org/XML/1998/namespace": "xml",
    "http://www.w3.org/1999/xhtml": "html",
    "http://www.w3.org/1999/02/22-rdf-syntax-ns#": "rdf",
    "http://schemas.xmlsoap.org/wsdl/": "wsdl",
    # xml schema
    "http://www.w3.org/2001/XMLSchema": "xs",
    "http://www.w3.org/2001/XMLSchema-instance": "xsi",
    # dublic core
    "http://purl.org/dc/elements/1.1/": "dc",
}


def _raise_serialization_error(text):  # pragma: no cover
    raise TypeError(
        "cannot serialize %r (type %s)" % (text, type(text).__name__)
        )


def _encode(text, encoding):
    try:
        return text.encode(encoding, "xmlcharrefreplace")
    except (TypeError, AttributeError):  # pragma: no cover
        _raise_serialization_error(text)


def _escape_cdata(text):
    # escape character data
    try:
        # it's worth avoiding do-nothing calls for strings that are
        # shorter than 500 character, or so.  assume that's, by far,
        # the most common case in most applications.
        if "&" in text:
            text = text.replace("&", "&amp;")
        if "<" in text:
            text = text.replace("<", "&lt;")
        if ">" in text:
            text = text.replace(">", "&gt;")
        return text
    except (TypeError, AttributeError):  # pragma: no cover
        _raise_serialization_error(text)


def _escape_attrib(text):
    # escape attribute value
    try:
        if "&" in text:
            text = text.replace("&", "&amp;")
        if "<" in text:
            text = text.replace("<", "&lt;")
        if ">" in text:
            text = text.replace(">", "&gt;")
        if "\"" in text:
            text = text.replace("\"", "&quot;")
        if "\n" in text:
            text = text.replace("\n", "&#10;")
        return text
    except (TypeError, AttributeError):  # pragma: no cover
        _raise_serialization_error(text)


def _escape_attrib_html(text):
    # escape attribute value
    try:
        if "&" in text:
            text = text.replace("&", "&amp;")
        if "<" in text:
            text = text.replace("<", "&lt;")
        if ">" in text:
            text = text.replace(">", "&gt;")
        if "\"" in text:
            text = text.replace("\"", "&quot;")
        return text
    except (TypeError, AttributeError):  # pragma: no cover
        _raise_serialization_error(text)


def _serialize_html(write, elem, qnames, namespaces, format):
    tag = elem.tag
    text = elem.text
    if tag is Comment:
        write("<!--%s-->" % _escape_cdata(text))
    elif tag is ProcessingInstruction:
        write("<?%s?>" % _escape_cdata(text))
    else:
        tag = qnames[tag]
        if tag is None:
            if text:
                write(_escape_cdata(text))
            for e in elem:
                _serialize_html(write, e, qnames, None, format)
        else:
            write("<" + tag)
            items = elem.items()
            if items or namespaces:
                items = sorted(items)  # lexical order
                for k, v in items:
                    if isinstance(k, QName):
                        k = k.text
                    if isinstance(v, QName):
                        v = qnames[v.text]
                    else:
                        v = _escape_attrib_html(v)
                    if qnames[k] == v and format == 'html':
                        # handle boolean attributes
                        write(" %s" % v)
                    else:
                        write(" %s=\"%s\"" % (qnames[k], v))
                if namespaces:
                    items = namespaces.items()
                    items.sort(key=lambda x: x[1])  # sort on prefix
                    for v, k in items:
                        if k:
                            k = ":" + k
                        write(" xmlns%s=\"%s\"" % (k, _escape_attrib(v)))
            if format == "xhtml" and tag.lower() in HTML_EMPTY:
                write(" />")
            else:
                write(">")
                if text:
                    if tag.lower() in ["script", "style"]:
                        write(text)
                    else:
                        write(_escape_cdata(text))
                for e in elem:
                    _serialize_html(write, e, qnames, None, format)
                if tag.lower() not in HTML_EMPTY:
                    write("</" + tag + ">")
    if elem.tail:
        write(_escape_cdata(elem.tail))


def _write_html(root,
                encoding=None,
                default_namespace=None,
                format="html"):
    assert root is not None
    data = []
    write = data.append
    qnames, namespaces = _namespaces(root, default_namespace)
    _serialize_html(write, root, qnames, namespaces, format)
    if encoding is None:
        return "".join(data)
    else:
        return _encode("".join(data))


# --------------------------------------------------------------------
# serialization support

def _namespaces(elem, default_namespace=None):
    # identify namespaces used in this tree

    # maps qnames to *encoded* prefix:local names
    qnames = {None: None}

    # maps uri:s to prefixes
    namespaces = {}
    if default_namespace:
        namespaces[default_namespace] = ""

    def add_qname(qname):
        # calculate serialized qname representation
        try:
            if qname[:1] == "{":
                uri, tag = qname[1:].split("}", 1)
                prefix = namespaces.get(uri)
                if prefix is None:
                    prefix = _namespace_map.get(uri)
                    if prefix is None:
                        prefix = "ns%d" % len(namespaces)
                    if prefix != "xml":
                        namespaces[uri] = prefix
                if prefix:
                    qnames[qname] = "%s:%s" % (prefix, tag)
                else:
                    qnames[qname] = tag  # default element
            else:
                if default_namespace:
                    raise ValueError(
                        "cannot use non-qualified names with "
                        "default_namespace option"
                        )
                qnames[qname] = qname
        except TypeError:  # pragma: no cover
            _raise_serialization_error(qname)

    # populate qname and namespaces table
    try:
        iterate = elem.iter
    except AttributeError:
        iterate = elem.getiterator  # cET compatibility
    for elem in iterate():
        tag = elem.tag
        if isinstance(tag, QName) and tag.text not in qnames:
            add_qname(tag.text)
        elif isinstance(tag, util.string_type):
            if tag not in qnames:
                add_qname(tag)
        elif tag is not None and tag is not Comment and tag is not PI:
            _raise_serialization_error(tag)
        for key, value in elem.items():
            if isinstance(key, QName):
                key = key.text
            if key not in qnames:
                add_qname(key)
            if isinstance(value, QName) and value.text not in qnames:
                add_qname(value.text)
        text = elem.text
        if isinstance(text, QName) and text.text not in qnames:
            add_qname(text.text)
    return qnames, namespaces


def to_html_string(element):
    return _write_html(ElementTree(element).getroot(), format="html")


def to_xhtml_string(element):
    return _write_html(ElementTree(element).getroot(), format="xhtml")



class HttpWebResponse(WebResponse,ISerializable,IDisposable):
 """
 Provides an HTTP-specific implementation of the System.Net.WebResponse class.

 

 HttpWebResponse()
 """
 def Close(self):
  """
  Close(self: HttpWebResponse)

   Closes the response stream.
  """
  pass
 def Dispose(self):
  """ Dispose(self: HttpWebResponse,disposing: bool) """
  pass
 def GetObjectData(self,*args):
  """
  GetObjectData(self: HttpWebResponse,serializationInfo: SerializationInfo,streamingContext: StreamingContext)

   Populates a System.Runtime.Serialization.SerializationInfo with the data needed to serialize the 

    target object.

  

  

   serializationInfo: The System.Runtime.Serialization.SerializationInfo to populate with data.

   streamingContext: A System.Runtime.Serialization.StreamingContext that specifies the destination for this 

    serialization.
  """
  pass
 def GetResponseHeader(self,headerName):
  """
  GetResponseHeader(self: HttpWebResponse,headerName: str) -> str

  

   Gets the contents of a header that was returned with the response.

  

   headerName: The header value to return.

   Returns: The contents of the specified header.
  """
  pass
 def GetResponseStream(self):
  """
  GetResponseStream(self: HttpWebResponse) -> Stream

  

   Gets the stream that is used to read the body of the response from the server.

   Returns: A System.IO.Stream containing the body of the response.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self):
  """
  __new__(cls: type)

  __new__(cls: type,serializationInfo: SerializationInfo,streamingContext: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 CharacterSet=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the character set of the response.



Get: CharacterSet(self: HttpWebResponse) -> str



"""

 ContentEncoding=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the method that is used to encode the body of the response.



Get: ContentEncoding(self: HttpWebResponse) -> str



"""

 ContentLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the length of the content returned by the request.



Get: ContentLength(self: HttpWebResponse) -> Int64



"""

 ContentType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the content type of the response.



Get: ContentType(self: HttpWebResponse) -> str



"""

 Cookies=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the cookies that are associated with this response.



Get: Cookies(self: HttpWebResponse) -> CookieCollection



Set: Cookies(self: HttpWebResponse)=value

"""

 Headers=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the headers that are associated with this response from the server.



Get: Headers(self: HttpWebResponse) -> WebHeaderCollection



"""

 IsMutuallyAuthenticated=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a System.Boolean value that indicates whether both client and server were authenticated.



Get: IsMutuallyAuthenticated(self: HttpWebResponse) -> bool



"""

 LastModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the last date and time that the contents of the response were modified.



Get: LastModified(self: HttpWebResponse) -> DateTime



"""

 Method=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the method that is used to return the response.



Get: Method(self: HttpWebResponse) -> str



"""

 ProtocolVersion=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the version of the HTTP protocol that is used in the response.



Get: ProtocolVersion(self: HttpWebResponse) -> Version



"""

 ResponseUri=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the URI of the Internet resource that responded to the request.



Get: ResponseUri(self: HttpWebResponse) -> Uri



"""

 Server=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the name of the server that sent the response.



Get: Server(self: HttpWebResponse) -> str



"""

 StatusCode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the status of the response.



Get: StatusCode(self: HttpWebResponse) -> HttpStatusCode



"""

 StatusDescription=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the status description returned with the response.



Get: StatusDescription(self: HttpWebResponse) -> str



"""

 SupportsHeaders=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: SupportsHeaders(self: HttpWebResponse) -> bool



"""





class ListViewHitTestInfo(object):
 """
 Contains information about an area of a System.Windows.Forms.ListView control or a System.Windows.Forms.ListViewItem.

 

 ListViewHitTestInfo(hitItem: ListViewItem,hitSubItem: ListViewSubItem,hitLocation: ListViewHitTestLocations)
 """
 def __getitem__(self,*args):
  """ x.__getitem__(y) <==> x[y] """
  pass
 @staticmethod
 def __new__(self,hitItem,hitSubItem,hitLocation):
  """ __new__(cls: type,hitItem: ListViewItem,hitSubItem: ListViewSubItem,hitLocation: ListViewHitTestLocations) """
  pass
 Item=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.ListViewItem at the position indicated by a hit test on a System.Windows.Forms.ListView.



Get: Item(self: ListViewHitTestInfo) -> ListViewItem



"""

 Location=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location of a hit test on a System.Windows.Forms.ListView control,in relation to the System.Windows.Forms.ListView and the items it contains.



Get: Location(self: ListViewHitTestInfo) -> ListViewHitTestLocations



"""

 SubItem=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.ListViewItem.ListViewSubItem at the position indicated by a hit test on a System.Windows.Forms.ListView.



Get: SubItem(self: ListViewHitTestInfo) -> ListViewSubItem



"""





"""Data structures for wrapping information returned by Revit server."""

import re
from collections import namedtuple
import datetime

import enum


ServerInfo = namedtuple('ServerInfo',
                        ['name',                   # type: str
                         'version',                # type: str
                         'machine_name',           # type: str
                         'roles',                  # type: list<ServerRole>
                         'access_level_types',     # type: list<str>
                         'max_path_length',        # type: int
                         'max_name_length',        # type: int
                         'servers',                # type: list<str>
                         ])
""" namedtuple for server properties

Attributes:
    name (str): Server name
    version (str): Revit Server version
    machine_name (str): Revit Server machine name
    roles (list<ServerRole>): List of server roles
    access_level_types (list<str>): List of access level types
    max_path_length (int): Maximum allowed length for server paths
    max_name_length (int): Maximum allowed length for server entry names
    servers (list<str>): List of neighbour servers
"""


class ServerRole(enum.Enum):
    """Enum representing various server role codes.

    Attributes:
        Host = 0
        Accelerator = 1
        Admin = 2
    """

    Host = 0
    Accelerator = 1
    Admin = 2


ServerDriveInfo = namedtuple('ServerDriveInfo',
                             ['drive_space',       # type: int
                              'drive_freespace',   # type: int
                              ])
""" namedtuple for server drive info

Attributes:
    drive_space (int): Total drive space in bytes
    drive_freespace (int): Free drive space in bytes
"""


EntryContents = namedtuple('EntryContents',
                           ['path',                # type: str
                            'drive_space',         # type: int
                            'drive_freespace',     # type: int
                            'files',               # type: list<FileInfo>
                            'folders',             # type: list<FolderInfo>
                            'lock_context',        # type: str
                            'lock_state',          # type: LockState
                            'locks_inprogress',    # type: list<IPLockInfo>
                            'models',              # type: list<ModelInfo>
                            ])
""" namedtuple for server entry contents
(Encapsulates result of /contents)

Attributes:
    path (str): Full path of this entry on server
    drive_space (int): Total server drive space in bytes
    drive_freespace (int): Free server drive space in bytes
    files (list<FileInfo>): List of all files under this entry
    folders (list<FolderInfo>): List of all folders under this entry
    lock_context (str): Lock context on this entry
    lock_state (LockState): Lock state on this entry
    locks_inprogress (list<IPLockInfo>): List of all in-progress locks
    models (list<ModelInfo>): List of all models under this entry
"""


EntryDirInfo = namedtuple('EntryDirInfo',
                          ['path',                # type: str
                           'name',                # type: str
                           'size',                # type: int
                           'date_created',        # type: DateEntry
                           'date_modified',       # type: DateEntry
                           'exists',              # type: bool
                           'folder_count',        # type: int
                           'is_folder',           # type: bool
                           'last_modified_by',    # type: str
                           'lock_context',        # type: str
                           'lock_state',          # type: LockState
                           'model_count',         # type: int
                           'model_size',          # type: int
                           'locks_inprogress',    # type: list<IPLockInfo>
                           ])
""" namedtuple for server entry directory info
(Encapsulates result of /directoryinfo)

Attributes:
    path (str): Full path of this entry on server
    name (str): Entry name including extension
    size (int): Entry size in bytes
    date_created (DateEntry): Date entry was created
    date_modified (DateEntry): Date entry was last modified
    exists (bool): True if entry exists
    folder_count (int): Number of sub folders
    is_folder (bool): True of this entry if a directory
    last_modified_by (str): Username of user who last modified the model
    lock_context (str): Lock context on this entry
    lock_state (LockState): Lock state on this entry
    model_count (int): Number of models under this entry
    model_size (int): If this entry is a model, size of model in bytes
    locks_inprogress (list<IPLockInfo>): List of all in-progress locks
"""


class LockState(enum.Enum):
    """Enum representing Revit Server lock states.

    Attributes:
        Unlocked = 0
        Locked = 1
        LockedParent = 2
        LockedChild = 3
        BeingUnlocked = 4
        BeingLocked = 5
    """

    Unlocked = 0
    Locked = 1
    LockedParent = 2
    LockedChild = 3
    BeingUnlocked = 4
    BeingLocked = 5


class LockOptions(enum.Enum):
    """Enum representing Revit Server lock options.

    Attributes:
        NotSet = 0
        Read = 1
        Write = 2
        NonExclusiveReadWrite = 128
        ReadAndNonExclusiveReadWrite = 129
        WriteAndNonExclusiveReadWrite = 130
        ReadWriteAndNonExclusiveReadWrite = 130
    """

    NotSet = 0
    Read = 1
    Write = 2
    NonExclusiveReadWrite = 128
    ReadAndNonExclusiveReadWrite = 129
    WriteAndNonExclusiveReadWrite = 130
    ReadWriteAndNonExclusiveReadWrite = 130


class LockType(enum.Enum):
    """Enum representing Revit Server lock type.

    Attributes:
        Data = 0
        Permission = 1
    """

    Data = 0
    Permissions = 1


FileInfo = namedtuple('FileInfo',
                      ['path',                     # type: str
                       'name',                     # type: str
                       'size',                     # type: int
                       'is_text',                  # type: bool
                       ])
""" namedtuple for info on a server file

Attributes:
    path (str): Full path of this file on server
    name (str): File name including extension
    size (int): File size in bytes
    is_text (bool): True if text file
"""


FolderInfo = namedtuple('FolderInfo',
                        ['path',                   # type: str
                         'name',                   # type: str
                         'size',                   # type: int
                         'has_contents',           # type: bool
                         'lock_context',           # type: str
                         'lock_state',             # type: LockState
                         'locks_inprogress',       # type: list<IPLockInfo>
                         ])
""" namedtuple for info on a server folder

Attributes:
    path (str): Full path of this folder on server
    name (str): File name including extension
    size (int): File size in bytes
    has_contents (bool): True if folder has contents
    lock_context (str): Lock context on this entry
    lock_state (LockState): Lock state on this entry
    locks_inprogress (list<IPLockInfo>): List of all in-progress locks
"""


ModelInfo = namedtuple('ModelInfo',
                       ['path',                    # type: str
                        'name',                    # type: str
                        'size',                    # type: int
                        'support_size',            # type: int
                        'product_version',         # type: int
                        'lock_context',            # type: str
                        'lock_state',              # type: LockState
                        'locks_inprogress',        # type: list<IPLockInfo>
                        ])
""" namedtuple for info on a server model

Attributes:
    path (str): Full path of this model on server
    name (str): File name including extension
    size (int): File size in bytes
    support_size (int): Size of all support files in bytes
    product_version (int): Revit version of this model
    lock_context (str): Lock context on this entry
    lock_state (LockState): Lock state on this entry
    locks_inprogress (list<IPLockInfo>): List of all in-progress locks
"""


ModelInfoEx = namedtuple('ModelInfoEx',
                         ['path',                  # type: str
                          'name',                  # type: str
                          'size',                  # type: int
                          'guid',                  # type: str
                          'date_created',          # type: DateEntry
                          'date_modified',         # type: DateEntry
                          'last_modified_by',      # type: str
                          'support_size',          # type: int
                          ])
""" namedtuple for extended info on a server model

Attributes:
    path (str): Full path of this model on server
    name (str): File name including extension
    size (int): File size in bytes
    guid (str): GUID of the model
    date_created (DateEntry): Date model was created
    date_modified (DateEntry): Date model was last modified
    last_modified_by (str): Username of user who last modified the model
    support_size (int): Size of all support files in bytes
"""


ProjectInfo = namedtuple('ProjectInfo',
                         ['parameters',            # type: list<ProjParameter>
                          ])
""" namedtuple for project info of a hosted model

Attributes:
    parameters (list<ProjParameter>): List of project parameters in this model
"""


ProjParameter = namedtuple('ProjParameter',
                           ['name',                # type: str
                            'value',               # type: str
                            'id',                  # type: str
                            'category',            # type: str
                            'type',                # type: ParamType
                            'datatype',            # type: ParamDataType
                            ])
""" namedtuple for info project parameters in a hosted model

Attributes:
    name (str): Parameter display name
    value (str): Parameter value
    id (str): Parameter unique id
    category (str): Parameter category name
    type (ParamType): Parameter Type
    datatype (ParamDataType): Parameter storage type
"""


class ParamType(enum.Enum):
    """Enum representing parameter types.

    Attributes:
        System = 'system'
        Custom = 'custom'
        Shared = 'shared'
        Unknown = 'unknown'
    """

    System = 'system'
    Custom = 'custom'
    Shared = 'shared'
    Unknown = 'unknown'


class ParamDataType(enum.Enum):
    """Enum representing parameter storage types.

    Attributes:
        Length = 'length'
        Number = 'number'
        Material = 'material'
        Text = 'text'
        MultilineText = "multiline text"
        YesNo = 'yes/no'
        Unknown = 'unknown'
    """

    Length = 'length'
    Number = 'number'
    Material = 'material'
    Text = 'text'
    MultilineText = "multiline text"
    YesNo = 'yes/no'
    Unknown = 'unknown'


MHistoryInfo = namedtuple('MHistoryInfo',
                          ['path',               # type: str
                           'items',              # type: list<MHistoryItemInfo>
                           ])
""" namedtuple for model history info

Attributes:
    path (str): Full path of the hosted model
    items (list<MHistoryItemInfo>): List of history items
"""


MHistoryItemInfo = namedtuple('MHistoryItemInfo',
                              ['id',               # type: str
                               'comment',          # type: str
                               'date',             # type: DateEntry
                               'model_size',       # type: int
                               'overwrittenby',    # type: str
                               'support_size',     # type: int
                               'user',             # type: str
                               ])
""" namedtuple for model history item info

Attributes:
    id (str): History item id
    comment (str): Comment recorded when syncing
    date (DateEntry): Date this history item was recorded
    size (int): File size in bytes
    overwrittenby (str): Id of history item overwriting this item
    support_size (int): Size of all support files in bytes
    user (str): Username recording the history item (by syncing model)
"""


IPLockInfo = namedtuple('IPLockInfo',
                        ['age',                    # type: TimeSpanEntry
                         'lock_options',           # type: LockOptions
                         'lock_type',              # type: LockType
                         'model_path',             # type: str
                         'timestamp',              # type: DateEntry
                         'username',               # type: str
                         ])
""" namedtuple for info on an in-progress entry lock

Attributes:
    age (str): Age of the lock
    lock_options (int): Lock options
    lock_type (int): Lock type
    model_path (str): Full path of the model being locked
    timestamp (DateEntry): Timestamp for when the lock was initiated
    username (str): Username who initiated this lock
"""


ChildrenLockInfo = namedtuple('ChildrenLockInfo',
                              ['path',             # type: str
                               'items',            # type: list<str>
                               'lock_context',     # type: str
                               ])
""" namedtuple for info locked children under an entry

Attributes:
    path (str): Full path of parent entry
    items (list<str>): List of paths of locked models under this entry
    lock_context (str): Lock context of parent entry
"""


class DateEntry(datetime.datetime):
    """Timestamp data type wrapping Revit Server string timestamps.

    Wraps Revit Server string timestamps in a typical python
    datetime.datetime subclass.

    Example:
        >>> ts = DateEntry.fromrsdatestring("/Date(1483465201000)/")
        DateEntry(2017, 1, 3, 17, 40, 1)
    """

    @classmethod
    def fromrsdatestring(cls, date_string):
        """Construct a class instance from Revit server timestamp.

        Args:
            date_string (str): Revit server timestamp string
        """
        seconds_since_epoch = int(date_string[6:-2])/1000
        return cls.utcfromtimestamp(seconds_since_epoch)


class TimeSpanEntry(datetime.timedelta):
    """Timespan data type wrapping Revit Server timespan.

    Wraps Revit Server string timespan in a typical python
    datetime.timedelta subclass

    Example:
        >>> ts = TimeSpanEntry.fromrstimespanstring("PT11M42.5154811S")
        TimeSpanEntry(0, 5856, 811000)
    """

    @classmethod
    def fromrstimespanstring(cls, timespan_string):
        """Construct a class instance from Revit server timespan.

        Args:
            timespan_string (str): Revit server timespan string
        """
        days = re.findall('(\d+)D', timespan_string)
        days = int(days[0]) if days else 0

        minutes = re.findall('(\d+)M', timespan_string)
        minutes = int(minutes[0]) if minutes else 0

        seconds = re.findall('(\d+)\.(\d+)S', timespan_string)
        seconds, millisecs = (int(seconds[0][0]), int(seconds[0][1])) \
            if seconds else (0, 0)

        return cls(days=days, minutes=minutes,
                   seconds=seconds, milliseconds=millisecs)



class Cookie(object):
 """
 Provides a set of properties and methods that are used to manage cookies. This class cannot be inherited.

 

 Cookie()

 Cookie(name: str,value: str)

 Cookie(name: str,value: str,path: str)

 Cookie(name: str,value: str,path: str,domain: str)
 """
 def Equals(self,comparand):
  """
  Equals(self: Cookie,comparand: object) -> bool

  

   Overrides the System.Object.Equals(System.Object) method.

  

   comparand: A reference to a System.Net.Cookie.

   Returns: Returns true if the System.Net.Cookie is equal to comparand. Two System.Net.Cookie instances are 

    equal if their System.Net.Cookie.Name,System.Net.Cookie.Value,System.Net.Cookie.Path,

    System.Net.Cookie.Domain,and System.Net.Cookie.Version properties are equal. 

    System.Net.Cookie.Name and System.Net.Cookie.Domain string comparisons are case-insensitive.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: Cookie) -> int

  

   Overrides the System.Object.GetHashCode method.

   Returns: The 32-bit signed integer hash code for this instance.
  """
  pass
 def ToString(self):
  """
  ToString(self: Cookie) -> str

  

   Overrides the System.Object.ToString method.

   Returns: Returns a string representation of this System.Net.Cookie object that is suitable for including 

    in a HTTP Cookie: request header.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 @staticmethod
 def __new__(self,name=None,value=None,path=None,domain=None):
  """
  __new__(cls: type)

  __new__(cls: type,name: str,value: str)

  __new__(cls: type,name: str,value: str,path: str)

  __new__(cls: type,name: str,value: str,path: str,domain: str)
  """
  pass
 def __ne__(self,*args):
  pass
 Comment=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a comment that the server can add to a System.Net.Cookie.



Get: Comment(self: Cookie) -> str



Set: Comment(self: Cookie)=value

"""

 CommentUri=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a URI comment that the server can provide with a System.Net.Cookie.



Get: CommentUri(self: Cookie) -> Uri



Set: CommentUri(self: Cookie)=value

"""

 Discard=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the discard flag set by the server.



Get: Discard(self: Cookie) -> bool



Set: Discard(self: Cookie)=value

"""

 Domain=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the URI for which the System.Net.Cookie is valid.



Get: Domain(self: Cookie) -> str



Set: Domain(self: Cookie)=value

"""

 Expired=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the current state of the System.Net.Cookie.



Get: Expired(self: Cookie) -> bool



Set: Expired(self: Cookie)=value

"""

 Expires=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the expiration date and time for the System.Net.Cookie as a System.DateTime.



Get: Expires(self: Cookie) -> DateTime



Set: Expires(self: Cookie)=value

"""

 HttpOnly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Determines whether a page script or other active content can access this cookie.



Get: HttpOnly(self: Cookie) -> bool



Set: HttpOnly(self: Cookie)=value

"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name for the System.Net.Cookie.



Get: Name(self: Cookie) -> str



Set: Name(self: Cookie)=value

"""

 Path=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the URIs to which the System.Net.Cookie applies.



Get: Path(self: Cookie) -> str



Set: Path(self: Cookie)=value

"""

 Port=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a list of TCP ports that the System.Net.Cookie applies to.



Get: Port(self: Cookie) -> str



Set: Port(self: Cookie)=value

"""

 Secure=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the security level of a System.Net.Cookie.



Get: Secure(self: Cookie) -> bool



Set: Secure(self: Cookie)=value

"""

 TimeStamp=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the time when the cookie was issued as a System.DateTime.



Get: TimeStamp(self: Cookie) -> DateTime



"""

 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Net.Cookie.Value for the System.Net.Cookie.



Get: Value(self: Cookie) -> str



Set: Value(self: Cookie)=value

"""

 Version=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the version of HTTP state maintenance to which the cookie conforms.



Get: Version(self: Cookie) -> int



Set: Version(self: Cookie)=value

"""





class PrintDialog(object):
 """
 Invokes a standard Microsoft Windows print dialog box that configures a System.Printing.PrintTicket and System.Printing.PrintQueue according to user input and then prints a document.

 

 PrintDialog()
 """
 def PrintDocument(self,documentPaginator,description):
  """
  PrintDocument(self: PrintDialog,documentPaginator: DocumentPaginator,description: str)

   Prints a System.Windows.Documents.DocumentPaginator object to the System.Printing.PrintQueue 

    that is currently selected.

  

  

   documentPaginator: The System.Windows.Documents.DocumentPaginator object to print.

   description: A description of the job that is to be printed. This text appears in the user interface (UI) of 

    the printer.
  """
  pass
 def PrintVisual(self,visual,description):
  """
  PrintVisual(self: PrintDialog,visual: Visual,description: str)

   Prints a visual (non-text) object,which is derived from the System.Windows.Media.Visual class,

    to the System.Printing.PrintQueue that is currently selected.

  

  

   visual: The System.Windows.Media.Visual to print.

   description: A description of the job that is to be printed. This text appears in the user interface (UI) of 

    the printer.
  """
  pass
 def ShowDialog(self):
  """
  ShowDialog(self: PrintDialog) -> Nullable[bool]

  

   Invokes the System.Windows.Controls.PrintDialog as a modal dialog box.

   Returns: true if a user clicks Print; false if a user clicks Cancel; or null if a user closes the dialog 

    box without clicking Print or Cancel.
  """
  pass
 CurrentPageEnabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: CurrentPageEnabled(self: PrintDialog) -> bool



Set: CurrentPageEnabled(self: PrintDialog)=value

"""

 MaxPage=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the highest page number that is allowed in page ranges.



Get: MaxPage(self: PrintDialog) -> UInt32



Set: MaxPage(self: PrintDialog)=value

"""

 MinPage=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the lowest page number that is allowed in page ranges.



Get: MinPage(self: PrintDialog) -> UInt32



Set: MinPage(self: PrintDialog)=value

"""

 PageRange=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the range of pages to print when System.Windows.Controls.PrintDialog.PageRangeSelection is set to System.Windows.Controls.PageRangeSelection.UserPages.



Get: PageRange(self: PrintDialog) -> PageRange



Set: PageRange(self: PrintDialog)=value

"""

 PageRangeSelection=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Windows.Controls.PageRangeSelection for this instance of System.Windows.Controls.PrintDialog.



Get: PageRangeSelection(self: PrintDialog) -> PageRangeSelection



Set: PageRangeSelection(self: PrintDialog)=value

"""

 PrintableAreaHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the printable area of the page.



Get: PrintableAreaHeight(self: PrintDialog) -> float



"""

 PrintableAreaWidth=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the width of the printable area of the page.



Get: PrintableAreaWidth(self: PrintDialog) -> float



"""

 PrintQueue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a System.Printing.PrintQueue that represents the printer that is selected.



Get: PrintQueue(self: PrintDialog) -> PrintQueue



Set: PrintQueue(self: PrintDialog)=value

"""

 PrintTicket=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the System.Printing.PrintTicket that is used by the System.Windows.Controls.PrintDialog when the user clicks Print for the current print job.



Get: PrintTicket(self: PrintDialog) -> PrintTicket



Set: PrintTicket(self: PrintDialog)=value

"""

 SelectedPagesEnabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Get: SelectedPagesEnabled(self: PrintDialog) -> bool



Set: SelectedPagesEnabled(self: PrintDialog)=value

"""

 UserPageRangeEnabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether users of the Print dialog box have the option to specify ranges of pages to print.



Get: UserPageRangeEnabled(self: PrintDialog) -> bool



Set: UserPageRangeEnabled(self: PrintDialog)=value

"""





class NullableConverter(TypeConverter):
 """
 Provides automatic conversion between a nullable type and its underlying primitive type.

 

 NullableConverter(type: Type)
 """
 def CanConvertFrom(self,*__args):
  """
  CanConvertFrom(self: NullableConverter,context: ITypeDescriptorContext,sourceType: Type) -> bool

  

   Returns whether this converter can convert an object of the given type to the type of this 

    converter,using the specified context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext  that provides a format context.

   sourceType: A System.Type that represents the type you want to convert from.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def CanConvertTo(self,*__args):
  """
  CanConvertTo(self: NullableConverter,context: ITypeDescriptorContext,destinationType: Type) -> bool

  

   Returns whether this converter can convert the object to the specified type,using the specified 

    context.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   destinationType: A System.Type that represents the type you want to convert to.

   Returns: true if this converter can perform the conversion; otherwise,false.
  """
  pass
 def ConvertFrom(self,*__args):
  """
  ConvertFrom(self: NullableConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object) -> object

  

   Converts the given object to the type of this converter,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: The System.Globalization.CultureInfo to use as the current culture.

   value: The System.Object to convert.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def ConvertTo(self,*__args):
  """
  ConvertTo(self: NullableConverter,context: ITypeDescriptorContext,culture: CultureInfo,value: object,destinationType: Type) -> object

  

   Converts the given value object to the specified type,using the specified context and culture 

    information.

  

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   culture: The System.Globalization.CultureInfo to use as the current culture.

   value: The System.Object to convert.

   destinationType: The System.Type to convert the value parameter to.

   Returns: An System.Object that represents the converted value.
  """
  pass
 def CreateInstance(self,*__args):
  """
  CreateInstance(self: NullableConverter,context: ITypeDescriptorContext,propertyValues: IDictionary) -> object

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   propertyValues: An System.Collections.IDictionary of new property values.

   Returns: An System.Object representing the given System.Collections.IDictionary,or null if the object 

    cannot be created. This method always returns null.
  """
  pass
 def GetCreateInstanceSupported(self,context=None):
  """
  GetCreateInstanceSupported(self: NullableConverter,context: ITypeDescriptorContext) -> bool

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if changing a property on this object requires a call to 

    System.ComponentModel.TypeConverter.CreateInstance(System.Collections.IDictionary) to create a 

    new value; otherwise,false.
  """
  pass
 def GetProperties(self,*__args):
  """
  GetProperties(self: NullableConverter,context: ITypeDescriptorContext,value: object,attributes: Array[Attribute]) -> PropertyDescriptorCollection

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: An System.Object that specifies the type of array for which to get properties.

   attributes: An array of type System.Attribute that is used as a filter.

   Returns: A System.ComponentModel.PropertyDescriptorCollection with the properties that are exposed for 

    this data type,or null if there are no properties.
  """
  pass
 def GetPropertiesSupported(self,context=None):
  """
  GetPropertiesSupported(self: NullableConverter,context: ITypeDescriptorContext) -> bool

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if System.ComponentModel.TypeConverter.GetProperties(System.Object) should be called to 

    find the properties of this object; otherwise,false.
  """
  pass
 def GetStandardValues(self,context=None):
  """
  GetStandardValues(self: NullableConverter,context: ITypeDescriptorContext) -> StandardValuesCollection

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context that can be used 

    to extract additional information about the environment from which this converter is invoked. 

    This parameter or properties of this parameter can be null.

  

   Returns: A System.ComponentModel.TypeConverter.StandardValuesCollection that holds a standard set of 

    valid values,or null if the data type does not support a standard set of values.
  """
  pass
 def GetStandardValuesExclusive(self,context=None):
  """
  GetStandardValuesExclusive(self: NullableConverter,context: ITypeDescriptorContext) -> bool

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if the System.ComponentModel.TypeConverter.StandardValuesCollection returned from 

    System.ComponentModel.TypeConverter.GetStandardValues is an exhaustive list of possible values; 

    false if other values are possible.
  """
  pass
 def GetStandardValuesSupported(self,context=None):
  """
  GetStandardValuesSupported(self: NullableConverter,context: ITypeDescriptorContext) -> bool

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   Returns: true if System.ComponentModel.TypeConverter.GetStandardValues should be called to find a common 

    set of values the object supports; otherwise,false.
  """
  pass
 def IsValid(self,*__args):
  """
  IsValid(self: NullableConverter,context: ITypeDescriptorContext,value: object) -> bool

  

   context: An System.ComponentModel.ITypeDescriptorContext that provides a format context.

   value: The System.Object to test for validity.
  """
  pass
 @staticmethod
 def __new__(self,type):
  """ __new__(cls: type,type: Type) """
  pass
 NullableType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the nullable type.



Get: NullableType(self: NullableConverter) -> Type



"""

 UnderlyingType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the underlying type.



Get: UnderlyingType(self: NullableConverter) -> Type



"""

 UnderlyingTypeConverter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the underlying type converter.



Get: UnderlyingTypeConverter(self: NullableConverter) -> TypeConverter



"""





class ToolStripPanelRenderEventArgs(EventArgs):
 """
 Provides data for System.Windows.Forms.ToolStripPanel drawing.

 

 ToolStripPanelRenderEventArgs(g: Graphics,toolStripPanel: ToolStripPanel)
 """
 @staticmethod
 def __new__(self,g,toolStripPanel):
  """ __new__(cls: type,g: Graphics,toolStripPanel: ToolStripPanel) """
  pass
 Graphics=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the graphics used to paint the System.Windows.Forms.ToolStripPanel.



Get: Graphics(self: ToolStripPanelRenderEventArgs) -> Graphics



"""

 Handled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether the event was handled.



Get: Handled(self: ToolStripPanelRenderEventArgs) -> bool



Set: Handled(self: ToolStripPanelRenderEventArgs)=value

"""

 ToolStripPanel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Forms.ToolStripPanel to paint.



Get: ToolStripPanel(self: ToolStripPanelRenderEventArgs) -> ToolStripPanel



"""





"""Provide access to classes and functionalty inside base loader module."""

from pyrevit.compat import IRONPY
from pyrevit.framework import clr
from pyrevit.runtime import RUNTIME_ASSM

#pylint: disable=import-error,invalid-name,broad-except,wildcard-import
# import base classes module
if IRONPY:
    clr.AddReference(RUNTIME_ASSM)
else:
    clr.AddReference(RUNTIME_ASSM.Location)

from PyRevitLabs.PyRevit.Runtime import *



class TraceListener(MarshalByRefObject,IDisposable):
 """ Provides the abstract base class for the listeners who monitor trace and debug output. """
 def Close(self):
  """
  Close(self: TraceListener)

   When overridden in a derived class,closes the output stream so it no longer receives tracing or 

    debugging output.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: TraceListener)

   Releases all resources used by the System.Diagnostics.TraceListener.
  """
  pass
 def Fail(self,message,detailMessage=None):
  """
  Fail(self: TraceListener,message: str,detailMessage: str)

   Emits an error message and a detailed error message to the listener you create when you 

    implement the System.Diagnostics.TraceListener class.

  

  

   message: A message to emit.

   detailMessage: A detailed message to emit.

  Fail(self: TraceListener,message: str)

   Emits an error message to the listener you create when you implement the 

    System.Diagnostics.TraceListener class.

  

  

   message: A message to emit.
  """
  pass
 def Flush(self):
  """
  Flush(self: TraceListener)

   When overridden in a derived class,flushes the output buffer.
  """
  pass
 def GetSupportedAttributes(self,*args):
  """
  GetSupportedAttributes(self: TraceListener) -> Array[str]

  

   Gets the custom attributes supported by the trace listener.

   Returns: A string array naming the custom attributes supported by the trace listener,or null if there 

    are no custom attributes.
  """
  pass
 def MemberwiseClone(self,*args):
  """
  MemberwiseClone(self: MarshalByRefObject,cloneIdentity: bool) -> MarshalByRefObject

  

   Creates a shallow copy of the current System.MarshalByRefObject object.

  

   cloneIdentity: false to delete the current System.MarshalByRefObject object's identity,which will cause the 

    object to be assigned a new identity when it is marshaled across a remoting boundary. A value of 

    false is usually appropriate. true to copy the current System.MarshalByRefObject object's 

    identity to its clone,which will cause remoting client calls to be routed to the remote server 

    object.

  

   Returns: A shallow copy of the current System.MarshalByRefObject object.

  MemberwiseClone(self: object) -> object

  

   Creates a shallow copy of the current System.Object.

   Returns: A shallow copy of the current System.Object.
  """
  pass
 def TraceData(self,eventCache,source,eventType,id,data):
  """
  TraceData(self: TraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,*data: Array[object])

   Writes trace information,an array of data objects and event information to the listener 

    specific output.

  

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   data: An array of objects to emit as data.

  TraceData(self: TraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,data: object)

   Writes trace information,a data object and event information to the listener specific output.

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   data: The trace data to emit.
  """
  pass
 def TraceEvent(self,eventCache,source,eventType,id,*__args):
  """
  TraceEvent(self: TraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,format: str,*args: Array[object])

   Writes trace information,a formatted array of objects and event information to the listener 

    specific output.

  

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   format: A format string that contains zero or more format items,which correspond to objects in the args 

    array.

  

   args: An object array containing zero or more objects to format.

  TraceEvent(self: TraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int,message: str)

   Writes trace information,a message,and event information to the listener specific output.

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.

   message: A message to write.

  TraceEvent(self: TraceListener,eventCache: TraceEventCache,source: str,eventType: TraceEventType,id: int)

   Writes trace and event information to the listener specific output.

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   eventType: One of the System.Diagnostics.TraceEventType values specifying the type of event that has caused 

    the trace.

  

   id: A numeric identifier for the event.
  """
  pass
 def TraceTransfer(self,eventCache,source,id,message,relatedActivityId):
  """
  TraceTransfer(self: TraceListener,eventCache: TraceEventCache,source: str,id: int,message: str,relatedActivityId: Guid)

   Writes trace information,a message,a related activity identity and event information to the 

    listener specific output.

  

  

   eventCache: A System.Diagnostics.TraceEventCache object that contains the current process ID,thread ID,and 

    stack trace information.

  

   source: A name used to identify the output,typically the name of the application that generated the 

    trace event.

  

   id: A numeric identifier for the event.

   message: A message to write.

   relatedActivityId: A System.Guid  object identifying a related activity.
  """
  pass
 def Write(self,*__args):
  """
  Write(self: TraceListener,message: str,category: str)

   Writes a category name and a message to the listener you create when you implement the 

    System.Diagnostics.TraceListener class.

  

  

   message: A message to write.

   category: A category name used to organize the output.

  Write(self: TraceListener,o: object,category: str)

   Writes a category name and the value of the object's System.Object.ToString method to the 

    listener you create when you implement the System.Diagnostics.TraceListener class.

  

  

   o: An System.Object whose fully qualified class name you want to write.

   category: A category name used to organize the output.

  Write(self: TraceListener,message: str)

   When overridden in a derived class,writes the specified message to the listener you create in 

    the derived class.

  

  

   message: A message to write.

  Write(self: TraceListener,o: object)

   Writes the value of the object's System.Object.ToString method to the listener you create when 

    you implement the System.Diagnostics.TraceListener class.

  

  

   o: An System.Object whose fully qualified class name you want to write.
  """
  pass
 def WriteIndent(self,*args):
  """
  WriteIndent(self: TraceListener)

   Writes the indent to the listener you create when you implement this class,and resets the 

    System.Diagnostics.TraceListener.NeedIndent property to false.
  """
  pass
 def WriteLine(self,*__args):
  """
  WriteLine(self: TraceListener,message: str,category: str)

   Writes a category name and a message to the listener you create when you implement the 

    System.Diagnostics.TraceListener class,followed by a line terminator.

  

  

   message: A message to write.

   category: A category name used to organize the output.

  WriteLine(self: TraceListener,o: object,category: str)

   Writes a category name and the value of the object's System.Object.ToString method to the 

    listener you create when you implement the System.Diagnostics.TraceListener class,followed by a 

    line terminator.

  

  

   o: An System.Object whose fully qualified class name you want to write.

   category: A category name used to organize the output.

  WriteLine(self: TraceListener,message: str)

   When overridden in a derived class,writes a message to the listener you create in the derived 

    class,followed by a line terminator.

  

  

   message: A message to write.

  WriteLine(self: TraceListener,o: object)

   Writes the value of the object's System.Object.ToString method to the listener you create when 

    you implement the System.Diagnostics.TraceListener class,followed by a line terminator.

  

  

   o: An System.Object whose fully qualified class name you want to write.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type)

  __new__(cls: type,name: str)
  """
  pass
 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the custom trace listener attributes defined in the application configuration file.



Get: Attributes(self: TraceListener) -> StringDictionary



"""

 Filter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets and sets the trace filter for the trace listener.



Get: Filter(self: TraceListener) -> TraceFilter



Set: Filter(self: TraceListener)=value

"""

 IndentLevel=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the indent level.



Get: IndentLevel(self: TraceListener) -> int



Set: IndentLevel(self: TraceListener)=value

"""

 IndentSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the number of spaces in an indent.



Get: IndentSize(self: TraceListener) -> int



Set: IndentSize(self: TraceListener)=value

"""

 IsThreadSafe=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the trace listener is thread safe.



Get: IsThreadSafe(self: TraceListener) -> bool



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a name for this System.Diagnostics.TraceListener.



Get: Name(self: TraceListener) -> str



Set: Name(self: TraceListener)=value

"""

 NeedIndent=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether to indent the output.



"""

 TraceOutputOptions=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the trace output options.



Get: TraceOutputOptions(self: TraceListener) -> TraceOptions



Set: TraceOutputOptions(self: TraceListener)=value

"""





class CurrencyWrapper(object):
 """
 Wraps objects the marshaler should marshal as a VT_CY.

 

 CurrencyWrapper(obj: Decimal)

 CurrencyWrapper(obj: object)
 """
 @staticmethod
 def __new__(self,obj):
  """
  __new__(cls: type,obj: Decimal)

  __new__(cls: type,obj: object)
  """
  pass
 WrappedObject=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the wrapped object to be marshaled as type VT_CY.



Get: WrappedObject(self: CurrencyWrapper) -> Decimal



"""





class DataReceivedEventArgs(EventArgs):
 """ Provides data for the System.Diagnostics.Process.OutputDataReceived and System.Diagnostics.Process.ErrorDataReceived events. """
 Data=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the line of characters that was written to a redirected System.Diagnostics.Process output stream.



Get: Data(self: DataReceivedEventArgs) -> str



"""





class _MethodInfo:
 """ Exposes the public members of the System.Reflection.MethodInfo class to unmanaged code. """
 def Equals(self,other):
  """
  Equals(self: _MethodInfo,other: object) -> bool

  

   Provides COM objects with version-independent access to the System.Object.Equals(System.Object) 

    method.

  

  

   other: The System.Object to compare with the current System.Object.

   Returns: true if the specified System.Object is equal to the current System.Object; otherwise,false.
  """
  pass
 def GetBaseDefinition(self):
  """
  GetBaseDefinition(self: _MethodInfo) -> MethodInfo

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MethodInfo.GetBaseDefinition method.

  

   Returns: A System.Reflection.MethodInfo object for the first implementation of this method.
  """
  pass
 def GetCustomAttributes(self,*__args):
  """
  GetCustomAttributes(self: _MethodInfo,inherit: bool) -> Array[object]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MemberInfo.GetCustomAttributes(System.Boolean) method.

  

  

   inherit: true to search this member's inheritance chain to find the attributes; otherwise,false.

   Returns: An array that contains all the custom attributes,or an array with zero (0) elements if no 

    attributes are defined.

  

  GetCustomAttributes(self: _MethodInfo,attributeType: Type,inherit: bool) -> Array[object]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MemberInfo.GetCustomAttributes(System.Type,System.Boolean) method.

  

  

   attributeType: The type of attribute to search for. Only attributes that are assignable to this type are 

    returned.

  

   inherit: true to search this member's inheritance chain to find the attributes; otherwise,false.

   Returns: An array of custom attributes applied to this member,or an array with zero (0) elements if no 

    attributes have been applied.
  """
  pass
 def GetHashCode(self):
  """
  GetHashCode(self: _MethodInfo) -> int

  

   Provides COM objects with version-independent access to the System.Object.GetHashCode method.

   Returns: The hash code for the current instance.
  """
  pass
 def GetIDsOfNames(self,riid,rgszNames,cNames,lcid,rgDispId):
  """
  GetIDsOfNames(self: _MethodInfo,riid: Guid,rgszNames: IntPtr,cNames: UInt32,lcid: UInt32,rgDispId: IntPtr) -> Guid

  

   Maps a set of names to a corresponding set of dispatch identifiers.

  

   riid: Reserved for future use. Must be IID_NULL.

   rgszNames: An array of names to be mapped.

   cNames: The count of the names to be mapped.

   lcid: The locale context in which to interpret the names.

   rgDispId: An array allocated by the caller that receives the identifiers corresponding to the names.
  """
  pass
 def GetMethodImplementationFlags(self):
  """
  GetMethodImplementationFlags(self: _MethodInfo) -> MethodImplAttributes

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MethodBase.GetMethodImplementationFlags method.

  

   Returns: One of the System.Reflection.MethodImplAttributes values.
  """
  pass
 def GetParameters(self):
  """
  GetParameters(self: _MethodInfo) -> Array[ParameterInfo]

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MethodBase.GetParameters method.

  

   Returns: An array of type System.Reflection.ParameterInfo containing information that matches the 

    signature of the method (or constructor) reflected by this instance.
  """
  pass
 def GetType(self):
  """
  GetType(self: _MethodInfo) -> Type

  

   Provides COM objects with version-independent access to the System.Type.GetType method.

   Returns: A System.Type object.
  """
  pass
 def GetTypeInfo(self,iTInfo,lcid,ppTInfo):
  """
  GetTypeInfo(self: _MethodInfo,iTInfo: UInt32,lcid: UInt32,ppTInfo: IntPtr)

   Retrieves the type information for an object,which can be used to get the type information for 

    an interface.

  

  

   iTInfo: The type information to return.

   lcid: The locale identifier for the type information.

   ppTInfo: A pointer to the requested type information object.
  """
  pass
 def GetTypeInfoCount(self,pcTInfo):
  """
  GetTypeInfoCount(self: _MethodInfo) -> UInt32

  

   Retrieves the number of type information interfaces that an object provides (either 0 or 1).
  """
  pass
 def Invoke(self,*__args):
  """
  Invoke(self: _MethodInfo,obj: object,parameters: Array[object]) -> object

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MethodBase.Invoke(System.Object,System.Object[]) method.

  

  

   obj: The instance that created this method.

   parameters: An argument list for the invoked method or constructor. This is an array of objects with the 

    same number,order,and type as the parameters of the method or constructor to be invoked. If 

    there are no parameters,parameters should be null.If the method or constructor represented by 

    this instance takes a ref parameter (ByRef in Visual Basic),no special attribute is required 

    for that parameter to invoke the method or constructor using this function. Any object in this 

    array that is not explicitly initialized with a value will contain the default value for that 

    object type. For reference type elements,this value is null. For value type elements,this 

    value is 0,0.0,or false,depending on the specific element type.

  

   Returns: An instance of the class associated with the constructor.

  Invoke(self: _MethodInfo,obj: object,invokeAttr: BindingFlags,binder: Binder,parameters: Array[object],culture: CultureInfo) -> object

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MethodBase.Invoke(System.Object,System.Reflection.BindingFlags,System.Reflectio

    n.Binder,System.Object[],System.Globalization.CultureInfo) method.

  

  

   obj: The instance that created this method.

   invokeAttr: One of the BindingFlags values that specifies the type of binding.

   binder: A Binder that defines a set of properties and enables the binding,coercion of argument types,

    and invocation of members using reflection. If binder is null,then Binder.DefaultBinding is 

    used.

  

   parameters: An array of type Object used to match the number,order,and type of the parameters for this 

    constructor,under the constraints of binder. If this constructor does not require parameters,

    pass an array with zero elements,as in Object[] parameters=new Object[0]. Any object in this 

    array that is not explicitly initialized with a value will contain the default value for that 

    object type. For reference type elements,this value is null. For value type elements,this 

    value is 0,0.0,or false,depending on the specific element type.

  

   culture: A System.Globalization.CultureInfo object used to govern the coercion of types. If this is null,

    the System.Globalization.CultureInfo for the current thread is used.

  

   Returns: An instance of the class associated with the constructor.

  Invoke(self: _MethodInfo,dispIdMember: UInt32,riid: Guid,lcid: UInt32,wFlags: Int16,pDispParams: IntPtr,pVarResult: IntPtr,pExcepInfo: IntPtr,puArgErr: IntPtr) -> Guid

  

   Provides access to properties and methods exposed by an object.

  

   dispIdMember: An identifier for the member.

   riid: Reserved for future use. Must be IID_NULL.

   lcid: The locale context in which to interpret arguments.

   wFlags: Flags describing the context of the call.

   pDispParams: A pointer to a structure containing an array of arguments,an array of argument DISPIDs for 

    named arguments,and counts for the number of elements in the arrays.

  

   pVarResult: A pointer to the location where the result will be stored.

   pExcepInfo: A pointer to a structure that contains exception information.

   puArgErr: The index of the first argument that has an error.
  """
  pass
 def IsDefined(self,attributeType,inherit):
  """
  IsDefined(self: _MethodInfo,attributeType: Type,inherit: bool) -> bool

  

   Provides COM objects with version-independent access to the 

    System.Reflection.MemberInfo.IsDefined(System.Type,System.Boolean) method.

  

  

   attributeType: The Type object to which the custom attributes are applied.

   inherit: true to search this member's inheritance chain to find the attributes; otherwise,false.

   Returns: true if one or more instance of the attributeType parameter is applied to this member; 

    otherwise,false.
  """
  pass
 def ToString(self):
  """
  ToString(self: _MethodInfo) -> str

  

   Provides COM objects with version-independent access to the System.Object.ToString method.

   Returns: A string that represents the current System.Object.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __str__(self,*args):
  pass
 Attributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.Attributes property.



Get: Attributes(self: _MethodInfo) -> MethodAttributes



"""

 CallingConvention=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.CallingConvention property.



Get: CallingConvention(self: _MethodInfo) -> CallingConventions



"""

 DeclaringType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.DeclaringType property.



Get: DeclaringType(self: _MethodInfo) -> Type



"""

 IsAbstract=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsAbstract property.



Get: IsAbstract(self: _MethodInfo) -> bool



"""

 IsAssembly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsAssembly property.



Get: IsAssembly(self: _MethodInfo) -> bool



"""

 IsConstructor=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsConstructor property.



Get: IsConstructor(self: _MethodInfo) -> bool



"""

 IsFamily=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsFamily property.



Get: IsFamily(self: _MethodInfo) -> bool



"""

 IsFamilyAndAssembly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsFamilyAndAssembly property.



Get: IsFamilyAndAssembly(self: _MethodInfo) -> bool



"""

 IsFamilyOrAssembly=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsFamilyOrAssembly property.



Get: IsFamilyOrAssembly(self: _MethodInfo) -> bool



"""

 IsFinal=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsFinal property.



Get: IsFinal(self: _MethodInfo) -> bool



"""

 IsHideBySig=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsHideBySig property.



Get: IsHideBySig(self: _MethodInfo) -> bool



"""

 IsPrivate=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsPrivate property.



Get: IsPrivate(self: _MethodInfo) -> bool



"""

 IsPublic=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsPublic property.



Get: IsPublic(self: _MethodInfo) -> bool



"""

 IsSpecialName=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsSpecialName property.



Get: IsSpecialName(self: _MethodInfo) -> bool



"""

 IsStatic=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsStatic property.



Get: IsStatic(self: _MethodInfo) -> bool



"""

 IsVirtual=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.IsVirtual property.



Get: IsVirtual(self: _MethodInfo) -> bool



"""

 MemberType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.MemberType property.



Get: MemberType(self: _MethodInfo) -> MemberTypes



"""

 MethodHandle=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodBase.MethodHandle property.



Get: MethodHandle(self: _MethodInfo) -> RuntimeMethodHandle



"""

 Name=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.Name property.



Get: Name(self: _MethodInfo) -> str



"""

 ReflectedType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MemberInfo.ReflectedType property.



Get: ReflectedType(self: _MethodInfo) -> Type



"""

 ReturnType=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodInfo.ReturnType property.



Get: ReturnType(self: _MethodInfo) -> Type



"""

 ReturnTypeCustomAttributes=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Provides COM objects with version-independent access to the System.Reflection.MethodInfo.ReturnTypeCustomAttributes property.



Get: ReturnTypeCustomAttributes(self: _MethodInfo) -> ICustomAttributeProvider



"""





class ICustomAdapter:
 """ Provides a way for clients to access the actual object,rather than the adapter object handed out by a custom marshaler. """
 def GetUnderlyingObject(self):
  """
  GetUnderlyingObject(self: ICustomAdapter) -> object

  

   Provides access to the underlying object wrapped by a custom marshaler.

   Returns: The object contained by the adapter object.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class DebuggerBrowsableAttribute(Attribute,_Attribute):
 """
 Determines if and how a member is displayed in the debugger variable windows. This class cannot be inherited.

 

 DebuggerBrowsableAttribute(state: DebuggerBrowsableState)
 """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,state):
  """ __new__(cls: type,state: DebuggerBrowsableState) """
  pass
 State=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the display state for the attribute.



Get: State(self: DebuggerBrowsableAttribute) -> DebuggerBrowsableState



"""





class IContainItemStorage:
 # no doc
 def Clear(self):
  """ Clear(self: IContainItemStorage) """
  pass
 def ClearItemValue(self,item,dp):
  """ ClearItemValue(self: IContainItemStorage,item: object,dp: DependencyProperty) """
  pass
 def ClearValue(self,dp):
  """ ClearValue(self: IContainItemStorage,dp: DependencyProperty) """
  pass
 def ReadItemValue(self,item,dp):
  """ ReadItemValue(self: IContainItemStorage,item: object,dp: DependencyProperty) -> object """
  pass
 def StoreItemValue(self,item,dp,value):
  """ StoreItemValue(self: IContainItemStorage,item: object,dp: DependencyProperty,value: object) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass



class ItemChangedEventArgs(EventArgs):
 """ Provides data for the System.Windows.Forms.CurrencyManager.ItemChanged event. """
 Index=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Indicates the position of the item being changed within the list.



Get: Index(self: ItemChangedEventArgs) -> int



"""





class DataGridViewRowHeightInfoPushedEventArgs(HandledEventArgs):
 """ Provides data for the System.Windows.Forms.DataGridView.RowHeightInfoPushed event of a System.Windows.Forms.DataGridView. """
 Height=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the height of the row the event occurred for.



Get: Height(self: DataGridViewRowHeightInfoPushedEventArgs) -> int



"""

 MinimumHeight=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the minimum height of the row the event occurred for.



Get: MinimumHeight(self: DataGridViewRowHeightInfoPushedEventArgs) -> int



"""

 RowIndex=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the index of the row the event occurred for.



Get: RowIndex(self: DataGridViewRowHeightInfoPushedEventArgs) -> int



"""





class ScrollEventArgs(EventArgs):
 """
 Provides data for the Scroll event.

 

 ScrollEventArgs(type: ScrollEventType,newValue: int)

 ScrollEventArgs(type: ScrollEventType,newValue: int,scroll: ScrollOrientation)

 ScrollEventArgs(type: ScrollEventType,oldValue: int,newValue: int)

 ScrollEventArgs(type: ScrollEventType,oldValue: int,newValue: int,scroll: ScrollOrientation)
 """
 @staticmethod
 def __new__(self,type,*__args):
  """
  __new__(cls: type,type: ScrollEventType,newValue: int)

  __new__(cls: type,type: ScrollEventType,newValue: int,scroll: ScrollOrientation)

  __new__(cls: type,type: ScrollEventType,oldValue: int,newValue: int)

  __new__(cls: type,type: ScrollEventType,oldValue: int,newValue: int,scroll: ScrollOrientation)
  """
  pass
 NewValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the new System.Windows.Forms.ScrollBar.Value of the scroll bar.



Get: NewValue(self: ScrollEventArgs) -> int



Set: NewValue(self: ScrollEventArgs)=value

"""

 OldValue=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the old System.Windows.Forms.ScrollBar.Value of the scroll bar.



Get: OldValue(self: ScrollEventArgs) -> int



"""

 ScrollOrientation=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the scroll bar orientation that raised the Scroll event.



Get: ScrollOrientation(self: ScrollEventArgs) -> ScrollOrientation



"""

 Type=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the type of scroll event that occurred.



Get: Type(self: ScrollEventArgs) -> ScrollEventType



"""





# encoding: utf-8
# module System.Windows.Documents calls itself Documents
# from PresentationCore,Version=4.0.0.0,Culture=neutral,PublicKeyToken=31bf3856ad364e35
# by generator 1.145
""" NamespaceTracker represent a CLS namespace. """
# no imports

# no functions
# classes

class ContentPosition(object):
 """ Abstract class that represents the position of content. This position is content specific. """
 Missing=None


class DocumentPage(object,IDisposable):
 """
 Represents a document page produced by a paginator.

 

 DocumentPage(visual: Visual)

 DocumentPage(visual: Visual,pageSize: Size,bleedBox: Rect,contentBox: Rect)
 """
 def Dispose(self):
  """
  Dispose(self: DocumentPage)

   Releases all resources used by the System.Windows.Documents.DocumentPage.
  """
  pass
 def OnPageDestroyed(self,*args):
  """
  OnPageDestroyed(self: DocumentPage,e: EventArgs)

   Raises the System.Windows.Documents.DocumentPage.PageDestroyed event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def SetBleedBox(self,*args):
  """
  SetBleedBox(self: DocumentPage,bleedBox: Rect)

   Sets the dimensions and location of the System.Windows.Documents.DocumentPage.BleedBox.

  

   bleedBox: An object that specifies the size and location of a rectangle.
  """
  pass
 def SetContentBox(self,*args):
  """
  SetContentBox(self: DocumentPage,contentBox: Rect)

   Sets the dimension and location of the System.Windows.Documents.DocumentPage.ContentBox.

  

   contentBox: An object that specifies the size and location of a rectangle.
  """
  pass
 def SetSize(self,*args):
  """
  SetSize(self: DocumentPage,size: Size)

   Sets the System.Windows.Documents.DocumentPage.Size of the physical page as it will be after any 

    cropping.

  

  

   size: The size of the page.
  """
  pass
 def SetVisual(self,*args):
  """
  SetVisual(self: DocumentPage,visual: Visual)

   Sets the System.Windows.Documents.DocumentPage.Visual that depicts the page.

  

   visual: The visual representation of the page.
  """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,visual,pageSize=None,bleedBox=None,contentBox=None):
  """
  __new__(cls: type,visual: Visual)

  __new__(cls: type,visual: Visual,pageSize: Size,bleedBox: Rect,contentBox: Rect)
  """
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 BleedBox=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the area for print production-related bleeds,registration marks,and crop marks that may appear on the physical sheet outside the logical page boundaries.



Get: BleedBox(self: DocumentPage) -> Rect



"""

 ContentBox=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the area of the page within the margins.



Get: ContentBox(self: DocumentPage) -> Rect



"""

 Size=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the actual size of a page as it will be following any cropping.



Get: Size(self: DocumentPage) -> Size



"""

 Visual=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets the visual representation of the page.



Get: Visual(self: DocumentPage) -> Visual



"""


 Missing=None
 PageDestroyed=None


class DocumentPaginator(object):
 """ Provides an abstract base class that supports creation of multiple-page elements from a single document. """
 def CancelAsync(self,userState):
  """
  CancelAsync(self: DocumentPaginator,userState: object)

   Cancels a previous erload:System.Windows.Documents.DocumentPaginator.GetPageAsync or 

    erload:System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberAsync operation.

  

  

   userState: The original userState passed to erload:System.Windows.Documents.DocumentPaginator.GetPageAsync,

    erload:System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberAsync,or 

    erload:System.Windows.Documents.DocumentPaginator.ComputePageCountAsync that identifies the 

    asynchronous task to cancel.
  """
  pass
 def ComputePageCount(self):
  """
  ComputePageCount(self: DocumentPaginator)

   Forces a pagination of the content,updates System.Windows.Documents.DocumentPaginator.PageCount 

    with the new total,and sets System.Windows.Documents.DocumentPaginator.IsPageCountValid to 

    true.
  """
  pass
 def ComputePageCountAsync(self,userState=None):
  """
  ComputePageCountAsync(self: DocumentPaginator,userState: object)

   Asynchronously,forces a pagination of the content,updates 

    System.Windows.Documents.DocumentPaginator.PageCount with the new total,sets 

    System.Windows.Documents.DocumentPaginator.IsPageCountValid to true.

  

  

   userState: A unique identifier for the asynchronous task.

  ComputePageCountAsync(self: DocumentPaginator)

   Asynchronously,forces a pagination of the content,updates 

    System.Windows.Documents.DocumentPaginator.PageCount with the new total,and sets 

    System.Windows.Documents.DocumentPaginator.IsPageCountValid to true.
  """
  pass
 def GetPage(self,pageNumber):
  """
  GetPage(self: DocumentPaginator,pageNumber: int) -> DocumentPage

  

   When overridden in a derived class,gets the System.Windows.Documents.DocumentPage for the 

    specified page number.

  

  

   pageNumber: The zero-based page number of the document page that is needed.

   Returns: The System.Windows.Documents.DocumentPage for the specified pageNumber,or 

    System.Windows.Documents.DocumentPage.Missing if the page does not exist.
  """
  pass
 def GetPageAsync(self,pageNumber,userState=None):
  """
  GetPageAsync(self: DocumentPaginator,pageNumber: int,userState: object)

   Asynchronously returns (through the System.Windows.Documents.DocumentPaginator.GetPageCompleted 

    event) the System.Windows.Documents.DocumentPage for the specified page number and assigns the 

    specified ID to the asynchronous task.

  

  

   pageNumber: The zero-based page number of the System.Windows.Documents.DocumentPage to get.

   userState: A unique identifier for the asynchronous task.

  GetPageAsync(self: DocumentPaginator,pageNumber: int)

   Asynchronously returns (through the System.Windows.Documents.DocumentPaginator.GetPageCompleted 

    event) the System.Windows.Documents.DocumentPage for the specified page number.

  

  

   pageNumber: The zero-based page number of the document page that is needed.
  """
  pass
 def OnComputePageCountCompleted(self,*args):
  """
  OnComputePageCountCompleted(self: DocumentPaginator,e: AsyncCompletedEventArgs)

   Raises the System.Windows.Documents.DocumentPaginator.ComputePageCountCompleted event.

  

   e: An System.ComponentModel.AsyncCompletedEventArgs that contains the event data.
  """
  pass
 def OnGetPageCompleted(self,*args):
  """
  OnGetPageCompleted(self: DocumentPaginator,e: GetPageCompletedEventArgs)

   Raises the System.Windows.Documents.DocumentPaginator.GetPageCompleted event.

  

   e: A System.Windows.Documents.GetPageCompletedEventArgs that contains the event data.
  """
  pass
 def OnPagesChanged(self,*args):
  """
  OnPagesChanged(self: DocumentPaginator,e: PagesChangedEventArgs)

   Raises the System.Windows.Documents.DocumentPaginator.PagesChanged event.

  

   e: A System.Windows.Documents.PagesChangedEventArgs that contains the event data.
  """
  pass
 IsPageCountValid=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets a value indicating whether System.Windows.Documents.DocumentPaginator.PageCount is the total number of pages.



Get: IsPageCountValid(self: DocumentPaginator) -> bool



"""

 PageCount=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets a count of the number of pages currently formatted



Get: PageCount(self: DocumentPaginator) -> int



"""

 PageSize=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,gets or sets the suggested width and height of each page.



Get: PageSize(self: DocumentPaginator) -> Size



Set: PageSize(self: DocumentPaginator)=value

"""

 Source=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When overridden in a derived class,returns the element being paginated.



Get: Source(self: DocumentPaginator) -> IDocumentPaginatorSource



"""


 ComputePageCountCompleted=None
 GetPageCompleted=None
 PagesChanged=None


class DynamicDocumentPaginator(DocumentPaginator):
 """ Provides an abstract base class that supports automatic background pagination and tracking content positions across repaginations in addition to the methods and properties of its own base class. """
 def GetObjectPosition(self,value):
  """
  GetObjectPosition(self: DynamicDocumentPaginator,value: object) -> ContentPosition

  

   When overridden in a derived class,returns a System.Windows.Documents.ContentPosition for the 

    specified System.Object.

  

  

   value: The object to return the System.Windows.Documents.ContentPosition of.

   Returns: The System.Windows.Documents.ContentPosition of the given object.
  """
  pass
 def GetPageNumber(self,contentPosition):
  """
  GetPageNumber(self: DynamicDocumentPaginator,contentPosition: ContentPosition) -> int

  

   When overridden in a derived class,returns the zero-based page number of the specified 

    System.Windows.Documents.ContentPosition.

  

  

   contentPosition: The content position whose page number is needed.

   Returns: An System.Int32 representing zero-based page number where the specified contentPosition appears.
  """
  pass
 def GetPageNumberAsync(self,contentPosition,userState=None):
  """
  GetPageNumberAsync(self: DynamicDocumentPaginator,contentPosition: ContentPosition,userState: object)

   Asynchronously,returns (through the This method raises the 

    System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberCompleted event) the zero-based 

    page number of the specified System.Windows.Documents.ContentPosition.

  

  

   contentPosition: The content position element to return the page number of.

   userState: A unique identifier for the asynchronous task.

  GetPageNumberAsync(self: DynamicDocumentPaginator,contentPosition: ContentPosition)

   Asynchronously,returns (through the This method raises the 

    System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberCompleted event) the zero-based 

    page number of the specified System.Windows.Documents.ContentPosition.

  

  

   contentPosition: The content position whose page number is needed.
  """
  pass
 def GetPagePosition(self,page):
  """
  GetPagePosition(self: DynamicDocumentPaginator,page: DocumentPage) -> ContentPosition

  

   When overridden in a derived class,gets the position of the specified page in the document's 

    content.

  

  

   page: The page whose position is needed.

   Returns: A System.Windows.Documents.ContentPosition representing the position of page.
  """
  pass
 def OnGetPageNumberCompleted(self,*args):
  """
  OnGetPageNumberCompleted(self: DynamicDocumentPaginator,e: GetPageNumberCompletedEventArgs)

   Raises the System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberCompleted event.

  

   e: A System.Windows.Documents.GetPageNumberCompletedEventArgs that contains the event data.
  """
  pass
 def OnPaginationCompleted(self,*args):
  """
  OnPaginationCompleted(self: DynamicDocumentPaginator,e: EventArgs)

   Raises the System.Windows.Documents.DynamicDocumentPaginator.PaginationCompleted event.

  

   e: An System.EventArgs that contains the event data.
  """
  pass
 def OnPaginationProgress(self,*args):
  """
  OnPaginationProgress(self: DynamicDocumentPaginator,e: PaginationProgressEventArgs)

   Raises the System.Windows.Documents.DynamicDocumentPaginator.PaginationProgress event.

  

   e: A System.Windows.Documents.PaginationProgressEventArgs that contains the event data.
  """
  pass
 IsBackgroundPaginationEnabled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value indicating whether pagination is performed automatically in the background in response to certain events,such as a change in page size.



Get: IsBackgroundPaginationEnabled(self: DynamicDocumentPaginator) -> bool



Set: IsBackgroundPaginationEnabled(self: DynamicDocumentPaginator)=value

"""


 GetPageNumberCompleted=None
 PaginationCompleted=None
 PaginationProgress=None


class GetPageCompletedEventArgs(AsyncCompletedEventArgs):
 """
 Provides data for the System.Windows.Documents.DocumentPaginator.GetPageCompleted event.

 

 GetPageCompletedEventArgs(page: DocumentPage,pageNumber: int,error: Exception,cancelled: bool,userState: object)
 """
 @staticmethod
 def __new__(self,page,pageNumber,error,cancelled,userState):
  """ __new__(cls: type,page: DocumentPage,pageNumber: int,error: Exception,cancelled: bool,userState: object) """
  pass
 DocumentPage=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Documents.DocumentPage for the page specified in the call to System.Windows.Documents.DocumentPaginator.GetPageAsync(System.Int32,System.Object).



Get: DocumentPage(self: GetPageCompletedEventArgs) -> DocumentPage



"""

 PageNumber=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the page number passed to System.Windows.Documents.DocumentPaginator.GetPageAsync(System.Int32,System.Object).



Get: PageNumber(self: GetPageCompletedEventArgs) -> int



"""



class GetPageCompletedEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that will handle the System.Windows.Documents.DocumentPaginator.GetPageCompleted event of a System.Windows.Documents.FixedDocument or other classes implementing System.Windows.Documents.DocumentPaginator.

 

 GetPageCompletedEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: GetPageCompletedEventHandler,sender: object,e: GetPageCompletedEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: GetPageCompletedEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: GetPageCompletedEventHandler,sender: object,e: GetPageCompletedEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class GetPageNumberCompletedEventArgs(AsyncCompletedEventArgs):
 """
 Provides data for the System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberCompleted event.

 

 GetPageNumberCompletedEventArgs(contentPosition: ContentPosition,pageNumber: int,error: Exception,cancelled: bool,userState: object)
 """
 @staticmethod
 def __new__(self,contentPosition,pageNumber,error,cancelled,userState):
  """ __new__(cls: type,contentPosition: ContentPosition,pageNumber: int,error: Exception,cancelled: bool,userState: object) """
  pass
 ContentPosition=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the System.Windows.Documents.ContentPosition passed to System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberAsync(System.Windows.Documents.ContentPosition).



Get: ContentPosition(self: GetPageNumberCompletedEventArgs) -> ContentPosition



"""

 PageNumber=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the page number for the System.Windows.Documents.ContentPosition passed to System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberAsync(System.Windows.Documents.ContentPosition).



Get: PageNumber(self: GetPageNumberCompletedEventArgs) -> int



"""



class GetPageNumberCompletedEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that will handle the System.Windows.Documents.DynamicDocumentPaginator.GetPageNumberCompleted event of a System.Windows.Documents.FixedDocument,or System.Windows.Documents.FlowDocument.

 

 GetPageNumberCompletedEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: GetPageNumberCompletedEventHandler,sender: object,e: GetPageNumberCompletedEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: GetPageNumberCompletedEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: GetPageNumberCompletedEventHandler,sender: object,e: GetPageNumberCompletedEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class IDocumentPaginatorSource:
 """ Defines the source object that performs actual content pagination. """
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 DocumentPaginator=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """When implemented in a derived class,gets the object that performs content pagination.



Get: DocumentPaginator(self: IDocumentPaginatorSource) -> DocumentPaginator



"""



class PagesChangedEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Documents.DocumentPaginator.PagesChanged event.

 

 PagesChangedEventArgs(start: int,count: int)
 """
 @staticmethod
 def __new__(self,start,count):
  """ __new__(cls: type,start: int,count: int) """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of continuous pages that changed.



Get: Count(self: PagesChangedEventArgs) -> int



"""

 Start=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the page number of the first page that changed.



Get: Start(self: PagesChangedEventArgs) -> int



"""



class PagesChangedEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that will handle the System.Windows.Documents.DocumentPaginator.PagesChanged event.

 

 PagesChangedEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: PagesChangedEventHandler,sender: object,e: PagesChangedEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: PagesChangedEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: PagesChangedEventHandler,sender: object,e: PagesChangedEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass

class PaginationProgressEventArgs(EventArgs):
 """
 Provides data for the System.Windows.Documents.DynamicDocumentPaginator.PaginationProgress event.

 

 PaginationProgressEventArgs(start: int,count: int)
 """
 @staticmethod
 def __new__(self,start,count):
  """ __new__(cls: type,start: int,count: int) """
  pass
 Count=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the number of continuous pages that were paginated.



Get: Count(self: PaginationProgressEventArgs) -> int



"""

 Start=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the page number of the first page that was paginated.



Get: Start(self: PaginationProgressEventArgs) -> int



"""



class PaginationProgressEventHandler(MulticastDelegate,ICloneable,ISerializable):
 """
 Represents the method that will handle the System.Windows.Documents.DynamicDocumentPaginator.PaginationProgress event.

 

 PaginationProgressEventHandler(object: object,method: IntPtr)
 """
 def BeginInvoke(self,sender,e,callback,object):
  """ BeginInvoke(self: PaginationProgressEventHandler,sender: object,e: PaginationProgressEventArgs,callback: AsyncCallback,object: object) -> IAsyncResult """
  pass
 def CombineImpl(self,*args):
  """
  CombineImpl(self: MulticastDelegate,follow: Delegate) -> Delegate

  

   Combines this System.Delegate with the specified System.Delegate to form a new delegate.

  

   follow: The delegate to combine with this delegate.

   Returns: A delegate that is the new root of the System.MulticastDelegate invocation list.
  """
  pass
 def DynamicInvokeImpl(self,*args):
  """
  DynamicInvokeImpl(self: Delegate,args: Array[object]) -> object

  

   Dynamically invokes (late-bound) the method represented by the current delegate.

  

   args: An array of objects that are the arguments to pass to the method represented by the current 

    delegate.-or- null,if the method represented by the current delegate does not require 

    arguments.

  

   Returns: The object returned by the method represented by the delegate.
  """
  pass
 def EndInvoke(self,result):
  """ EndInvoke(self: PaginationProgressEventHandler,result: IAsyncResult) """
  pass
 def GetMethodImpl(self,*args):
  """
  GetMethodImpl(self: MulticastDelegate) -> MethodInfo

  

   Returns a static method represented by the current System.MulticastDelegate.

   Returns: A static method represented by the current System.MulticastDelegate.
  """
  pass
 def Invoke(self,sender,e):
  """ Invoke(self: PaginationProgressEventHandler,sender: object,e: PaginationProgressEventArgs) """
  pass
 def RemoveImpl(self,*args):
  """
  RemoveImpl(self: MulticastDelegate,value: Delegate) -> Delegate

  

   Removes an element from the invocation list of this System.MulticastDelegate that is equal to 

    the specified delegate.

  

  

   value: The delegate to search for in the invocation list.

   Returns: If value is found in the invocation list for this instance,then a new System.Delegate without 

    value in its invocation list; otherwise,this instance with its original invocation list.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,object,method):
  """ __new__(cls: type,object: object,method: IntPtr) """
  pass
 def __reduce_ex__(self,*args):
  pass




class SourceFilter(TraceFilter):
 """
 Indicates whether a listener should trace a message based on the source of a trace.

 

 SourceFilter(source: str)
 """
 def ShouldTrace(self,cache,source,eventType,id,formatOrMessage,args,data1,data):
  """
  ShouldTrace(self: SourceFilter,cache: TraceEventCache,source: str,eventType: TraceEventType,id: int,formatOrMessage: str,args: Array[object],data1: object,data: Array[object]) -> bool

  

   Determines whether the trace listener should trace the event.

  

   cache: An object that represents the information cache for the trace event.

   source: The name of the source.

   eventType: One of the enumeration values that identifies the event type.

   id: A trace identifier number.

   formatOrMessage: The format to use for writing an array of arguments or a message to write.

   args: An array of argument objects.

   data1: A trace data object.

   data: An array of trace data objects.

   Returns: true if the trace should be produced; otherwise,false.
  """
  pass
 @staticmethod
 def __new__(self,source):
  """ __new__(cls: type,source: str) """
  pass
 Source=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the name of the trace source.



Get: Source(self: SourceFilter) -> str



Set: Source(self: SourceFilter)=value

"""





""" Assembly Wrappers """

from rpw import revit, DB
from rpw.db.element import Element
from rpw.db.family import FamilyInstance, FamilySymbol

from rpw.utils.coerce import to_elements
from rpw.utils.mixins import CategoryMixin


# TODO: Tests
# TODO: Inherit from FamilyInstance Instead

class AssemblyInstance(Element, CategoryMixin):
    """
    `DB.AssemblyInstance` Wrapper

    Attribute:
        _revit_object (DB.AssemblyInstance): Wrapped ``DB.AssemblyInstance``
    """

    _revit_object_class = DB.AssemblyInstance
    _collector_params = {'of_class': _revit_object_class, 'is_type': False}

    @property
    def symbol(self):
        """ Alias to AssemblyInstance.type """
        return self.type

    def get_elements(self, wrapped=True):
        """
        Get other elements inside parent assembly.

        Returns:
            other elements inside the assembly

        """
        member_ids = self._revit_object.GetMemberIds()
        elements = to_elements(member_ids, doc=self._revit_object.Document)
        return [Element(e) if wrapped else e for e in elements]

    def __repr__(self):
        return Element.__repr__(self, data={'name': self.Name})


class AssemblyType(FamilySymbol, CategoryMixin):
    """
    `DB.AssemblyType` Wrapper
    Inherits from :any:`Element`

    Attribute:
        _revit_object (DB.AssemblyType): Wrapped ``DB.AssemblyType``
    """

    _revit_object_class = DB.AssemblyType
    _collector_params = {'of_class': _revit_object_class, 'is_type': True}

    @property
    def siblings(self):
        """ Returns all assembly types """
        return [Element.from_id(t) for t in self._revit_object.GetSimilarTypes()]

    @property
    def instances(self):
        raise NotImplemented
        """ Returns all Instances of the assembly type """
        # bip = BipEnum.get_id('AREA_SCHEME_ID')
        # param_filter = rpw.db.Collector.ParameterFilter(bip, equals=self._revit_object.Id)
        # collector = rpw.db.Collector(parameter_filter=param_filter,
        #                              **Area._collector_params)
        # return collector.wrapped_elements


    def __repr__(self):
        return Element.__repr__(self, data={'name': self.name})



class DoubleAnimationUsingPath(DoubleAnimationBase,ISealable,IAnimatable,IResource):
 """
 Animates the value of a System.Double property between two or more target values using a System.Windows.Media.PathGeometry to specify those values. This animation can be used to move a visual object along a path.

 

 DoubleAnimationUsingPath()
 """
 def AllocateClock(self,*args):
  """
  AllocateClock(self: AnimationTimeline) -> Clock

  

   Creates a System.Windows.Media.Animation.Clock for this 

    System.Windows.Media.Animation.AnimationTimeline.

  

   Returns: A clock for this System.Windows.Media.Animation.AnimationTimeline.
  """
  pass
 def Clone(self):
  """
  Clone(self: DoubleAnimationUsingPath) -> DoubleAnimationUsingPath

  

   Creates a modifiable clone of this System.Windows.Media.Animation.DoubleAnimationUsingPath,

    making deep copies of this object's values. When copying dependency properties,this method 

    copies resource references and data bindings (but they might no longer resolve) but not 

    animations or their current values.

  

   Returns: A modifiable clone of the current object. The cloned object's System.Windows.Freezable.IsFrozen 

    property will be false even if the source's System.Windows.Freezable.IsFrozen property was true.
  """
  pass
 def CloneCore(self,*args):
  """
  CloneCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a clone (deep copy) of the specified System.Windows.Freezable using base 

    (non-animated) property values.

  

  

   sourceFreezable: The object to clone.
  """
  pass
 def CloneCurrentValueCore(self,*args):
  """
  CloneCurrentValueCore(self: Freezable,sourceFreezable: Freezable)

   Makes the instance a modifiable clone (deep copy) of the specified System.Windows.Freezable 

    using current property values.

  

  

   sourceFreezable: The System.Windows.Freezable to be cloned.
  """
  pass
 def CreateInstance(self,*args):
  """
  CreateInstance(self: Freezable) -> Freezable

  

   Initializes a new instance of the System.Windows.Freezable class.

   Returns: The new instance.
  """
  pass
 def CreateInstanceCore(self,*args):
  """
  CreateInstanceCore(self: DoubleAnimationUsingPath) -> Freezable

  

   Implementation of System.Windows.Freezable.CreateInstanceCore.

   Returns: The new System.Windows.Freezable created.
  """
  pass
 def FreezeCore(self,*args):
  """
  FreezeCore(self: Timeline,isChecking: bool) -> bool

  

   Makes this System.Windows.Media.Animation.Timeline unmodifiable or determines whether it can be 

    made unmodifiable.

  

  

   isChecking: true to check if this instance can be frozen; false to freeze this instance.

   Returns: If isChecking is true,this method returns true if this instance can be made read-only,or false 

    if it cannot be made read-only. If isChecking is false,this method returns true if this 

    instance is now read-only,or false if it cannot be made read-only,with the side effect of 

    having begun to change the frozen status of this object.
  """
  pass
 def GetAsFrozenCore(self,*args):
  """
  GetAsFrozenCore(self: Timeline,sourceFreezable: Freezable)

   Makes this instance a clone of the specified System.Windows.Media.Animation.Timeline object.

  

   sourceFreezable: The System.Windows.Media.Animation.Timeline instance to clone.
  """
  pass
 def GetCurrentValueAsFrozenCore(self,*args):
  """
  GetCurrentValueAsFrozenCore(self: Timeline,sourceFreezable: Freezable)

   Makes this instance a frozen clone of the specified System.Windows.Media.Animation.Timeline. 

    Resource references,data bindings,and animations are not copied,but their current values are.

  

  

   sourceFreezable: The System.Windows.Media.Animation.Timeline to copy and freeze.
  """
  pass
 def GetCurrentValueCore(self,*args):
  """
  GetCurrentValueCore(self: DoubleAnimationUsingPath,defaultOriginValue: float,defaultDestinationValue: float,animationClock: AnimationClock) -> float

  

   Calculates a value that represents the current value of the property being animated,as 

    determined by the System.Windows.Media.Animation.DoubleAnimationUsingPath.

  

  

   defaultOriginValue: The suggested origin value,used if the animation does not have its own explicitly set start 

    value.

  

   defaultDestinationValue: The suggested destination value,used if the animation does not have its own explicitly set end 

    value.

  

   animationClock: An System.Windows.Media.Animation.AnimationClock that generates the 

    System.Windows.Media.Animation.Clock.CurrentTime or 

    System.Windows.Media.Animation.Clock.CurrentProgress used by the animation.

  

   Returns: The calculated value of the property,as determined by the current animation.
  """
  pass
 def GetNaturalDuration(self,*args):
  """
  GetNaturalDuration(self: Timeline,clock: Clock) -> Duration

  

   Returns the length of a single iteration of this System.Windows.Media.Animation.Timeline.

  

   clock: The System.Windows.Media.Animation.Clock that was created for this 

    System.Windows.Media.Animation.Timeline.

  

   Returns: The length of a single iteration of this System.Windows.Media.Animation.Timeline,or 

    System.Windows.Duration.Automatic if the natural duration is unknown.
  """
  pass
 def GetNaturalDurationCore(self,*args):
  """
  GetNaturalDurationCore(self: AnimationTimeline,clock: Clock) -> Duration

  

   Returns the length of a single iteration of this 

    System.Windows.Media.Animation.AnimationTimeline.

  

  

   clock: The clock that was created for this System.Windows.Media.Animation.AnimationTimeline.

   Returns: The animation's natural duration. This method always returns a System.Windows.Duration of 1 

    second.
  """
  pass
 def OnChanged(self,*args):
  """
  OnChanged(self: DoubleAnimationUsingPath)

   Called when this System.Windows.Media.Animation.DoubleAnimationUsingPath is modified.
  """
  pass
 def OnFreezablePropertyChanged(self,*args):
  """
  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject,property: DependencyProperty)

   This member supports the Windows Presentation Foundation (WPF) infrastructure and is not 

    intended to be used directly from your code.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.

   property: The property that changed.

  OnFreezablePropertyChanged(self: Freezable,oldValue: DependencyObject,newValue: DependencyObject)

   Ensures that appropriate context pointers are established for a 

    System.Windows.DependencyObjectType data member that has just been set.

  

  

   oldValue: The previous value of the data member.

   newValue: The current value of the data member.
  """
  pass
 def OnPropertyChanged(self,*args):
  """
  OnPropertyChanged(self: Freezable,e: DependencyPropertyChangedEventArgs)

   Overrides the System.Windows.DependencyObject implementation of 

    System.Windows.DependencyObject.OnPropertyChanged(System.Windows.DependencyPropertyChangedEventAr

    gs) to also invoke any System.Windows.Freezable.Changed handlers in response to a changing 

    dependency property of type System.Windows.Freezable.

  

  

   e: Event data that contains information about which property changed,and its old and new values.
  """
  pass
 def ReadPreamble(self,*args):
  """
  ReadPreamble(self: Freezable)

   Ensures that the System.Windows.Freezable is being accessed from a valid thread. Inheritors of 

    System.Windows.Freezable must call this method at the beginning of any API that reads data 

    members that are not dependency properties.
  """
  pass
 def ShouldSerializeProperty(self,*args):
  """
  ShouldSerializeProperty(self: DependencyObject,dp: DependencyProperty) -> bool

  

   Returns a value that indicates whether serialization processes should serialize the value for 

    the provided dependency property.

  

  

   dp: The identifier for the dependency property that should be serialized.

   Returns: true if the dependency property that is supplied should be value-serialized; otherwise,false.
  """
  pass
 def WritePostscript(self,*args):
  """
  WritePostscript(self: Freezable)

   Raises the System.Windows.Freezable.Changed event for the System.Windows.Freezable and invokes 

    its System.Windows.Freezable.OnChanged method. Classes that derive from System.Windows.Freezable 

    should call this method at the end of any API that modifies class members that are not stored as 

    dependency properties.
  """
  pass
 def WritePreamble(self,*args):
  """
  WritePreamble(self: Freezable)

   Verifies that the System.Windows.Freezable is not frozen and that it is being accessed from a 

    valid threading context. System.Windows.Freezable inheritors should call this method at the 

    beginning of any API that writes to data members that are not dependency properties.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 IsAdditive=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that indicates whether the target property's current value should be added to this animation's starting value.



Get: IsAdditive(self: DoubleAnimationUsingPath) -> bool



Set: IsAdditive(self: DoubleAnimationUsingPath)=value

"""

 IsCumulative=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a value that specifies whether the animation's value accumulates when it repeats.



Get: IsCumulative(self: DoubleAnimationUsingPath) -> bool



Set: IsCumulative(self: DoubleAnimationUsingPath)=value

"""

 PathGeometry=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Specifies the geometry used to generate this animation's output values.



Get: PathGeometry(self: DoubleAnimationUsingPath) -> PathGeometry



Set: PathGeometry(self: DoubleAnimationUsingPath)=value

"""

 Source=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the aspect of this animation's System.Windows.Media.Animation.DoubleAnimationUsingPath.PathGeometry that determines its output value.



Get: Source(self: DoubleAnimationUsingPath) -> PathAnimationSource



Set: Source(self: DoubleAnimationUsingPath)=value

"""


 PathGeometryProperty=None
 SourceProperty=None




"""Revit application wrapper."""
import types
import sys

from pyrevit import HOST_APP, DOCS
from pyrevit import PyRevitException
from pyrevit import framework
from pyrevit.coreutils.logger import get_logger
from pyrevit import DB, UI

#pylint: disable=W0401
from pyrevit.revit.db import *
from pyrevit.revit.db import query
from pyrevit.revit.db import select
from pyrevit.revit.db import create
from pyrevit.revit.db import update
from pyrevit.revit.db import ensure
from pyrevit.revit.db import delete
from pyrevit.revit.db.transaction import *
from pyrevit.revit.db import failure
from pyrevit.revit.db.pickling import *
from pyrevit.revit.journals import *
from pyrevit.revit.selection import *
from pyrevit.revit import ui
from pyrevit.revit import tabs
from pyrevit.revit import events
from pyrevit.revit import report
from pyrevit.revit import files
from pyrevit.revit import serverutils
from pyrevit.revit import geom
from pyrevit.revit import units
from pyrevit.revit import features
from pyrevit.revit import bim360
from pyrevit.revit import dc3dserver


#pylint: disable=W0703,C0302,C0103
mlogger = get_logger(__name__)


def get_imported_symbol(symbol_name):
    """Geth an imported symbol by its name.

    Args:
        symbol_name (str): symbol name

    Returns:
        (Any): imported symbol, if found, None otherwise.
    """
    return globals().get(symbol_name, None)


class RevitWrapper(types.ModuleType):
    """Revit application wrapper."""
    def __init__(self):
        pass

    def __getattribute__(self, attr_name):
        attr = get_imported_symbol(attr_name)
        return attr or object.__getattribute__(self, attr_name)

    def __getattr__(self, attr_name):
        attr = get_imported_symbol(attr_name)
        if not attr:
            raise AttributeError('\'module\' object has no attribute \'{}\''
                                 .format(attr_name))
        return attr

    @property
    def uidoc(self):
        """Active UI Document."""
        return HOST_APP.uidoc

    @property
    def doc(self):
        """Active document."""
        return DOCS.doc

    @property
    def docs(self):
        """Active documents."""
        return DOCS.docs

    @property
    def active_view(self):
        """Active view."""
        return HOST_APP.active_view

    @active_view.setter
    def active_view(self, value):
        HOST_APP.active_view = value

    @property
    def active_ui_view(self):
        """Active UI view."""
        if isinstance(self.active_view, DB.View):
            for uiview in self.uidoc.GetOpenUIViews():
                if uiview.ViewId == self.active_view.Id:
                    return uiview

    @property
    def servers(self):
        """Available revit server names."""
        return HOST_APP.available_servers

    @staticmethod
    def open_doc(doc_path):
        """Open document at given path.

        Args:
            doc_path (str): document file path

        Returns:
            (DB.Document): opened document
        """
        return HOST_APP.app.OpenDocumentFile(doc_path)

    @staticmethod
    def close_doc(doc):
        """Close given document.

        Args:
            doc (DB.Document): document
        """
        return doc.Close()

    @staticmethod
    def post_command(command_id):
        """Request Revit to run a command.

        Args:
            command_id (str): command identifier e.g. ID_REVIT_SAVE_AS_TEMPLATE
        """
        HOST_APP.post_command(command_id)


class ErrorSwallower():
    """Suppresses warnings during script execution.

    Examples:
        ```python
        with ErrorSwallower() as swallower:
            for fam in families:
                revit.doc.EditFamily(fam)
                if swallower.get_swallowed():
                    logger.warn("Warnings swallowed")
        ```
    """

    def __init__(self, log_errors=True):
        self._fswallower = failure.FailureSwallower()
        self._logerror = log_errors

    def on_failure_processing(self, _, event_args):
        """Failure processing event handler."""
        try:
            failure_accesssor = event_args.GetFailuresAccessor()
            mlogger.debug('request for failure processing...')
            result = event_args.GetProcessingResult()
            mlogger.debug('current failure processing result: %s', result)
            result = self._fswallower.preprocess_failures(failure_accesssor)
            mlogger.debug('setting failure processing results to: %s', result)
            event_args.SetProcessingResult(result)
        except Exception as fpex:
            mlogger.error('Error occurred while processing failures. | %s', fpex)

    def get_swallowed_errors(self):
        """Return swallowed errors."""
        return self._fswallower.get_swallowed_failures()

    def reset(self):
        """Reset swallowed errors."""
        self._fswallower.reset()

    def __enter__(self):
        """Start listening to failure processing events."""
        HOST_APP.app.FailuresProcessing += self.on_failure_processing
        return self

    def __exit__(self, exception, exception_value, traceback):
        """Stop listening to failure processing events."""
        HOST_APP.app.FailuresProcessing -= self.on_failure_processing
        if exception and self._logerror:
            mlogger.error('Error in ErrorSwallower Context. | %s:%s',
                          exception, exception_value)


sys.modules[__name__] = RevitWrapper()



class Win32Exception(ExternalException,ISerializable,_Exception):
 """
 Throws an exception for a Win32 error code.

 

 Win32Exception()

 Win32Exception(error: int)

 Win32Exception(error: int,message: str)

 Win32Exception(message: str)

 Win32Exception(message: str,innerException: Exception)
 """
 def add_SerializeObjectState(self,*args):
  """ add_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: Win32Exception,info: SerializationInfo,context: StreamingContext)

   Sets the System.Runtime.Serialization.SerializationInfo object with the file name and line 

    number at which this System.ComponentModel.Win32Exception occurred.

  

  

   info: A System.Runtime.Serialization.SerializationInfo.

   context: The contextual information about the source or destination.
  """
  pass
 def remove_SerializeObjectState(self,*args):
  """ remove_SerializeObjectState(self: Exception,value: EventHandler[SafeSerializationEventArgs]) """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*__args):
  """
  __new__(cls: type)

  __new__(cls: type,error: int)

  __new__(cls: type,error: int,message: str)

  __new__(cls: type,message: str)

  __new__(cls: type,message: str,innerException: Exception)

  __new__(cls: type,info: SerializationInfo,context: StreamingContext)
  """
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 NativeErrorCode=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the Win32 error code associated with this exception.



Get: NativeErrorCode(self: Win32Exception) -> int



"""



# variables with complex values




class RuntimeFieldHandle(object,ISerializable):
 """ Represents a field using an internal metadata token. """
 def Equals(self,*__args):
  """
  Equals(self: RuntimeFieldHandle,handle: RuntimeFieldHandle) -> bool

  

   Indicates whether the current instance is equal to the specified System.RuntimeFieldHandle.

  

   handle: The System.RuntimeFieldHandle to compare to the current instance.

   Returns: true if the value of handle is equal to the value of the current instance; otherwise,false.

  Equals(self: RuntimeFieldHandle,obj: object) -> bool

  

   Indicates whether the current instance is equal to the specified object.

  

   obj: The object to compare to the current instance.

   Returns: true if obj is a System.RuntimeFieldHandle and equal to the value of the current instance; 

    otherwise,false.
  """
  pass
 def GetHashCode(self):
  """ GetHashCode(self: RuntimeFieldHandle) -> int """
  pass
 def GetObjectData(self,info,context):
  """
  GetObjectData(self: RuntimeFieldHandle,info: SerializationInfo,context: StreamingContext)

   Populates a System.Runtime.Serialization.SerializationInfo with the data necessary to 

    deserialize the field represented by the current instance.

  

  

   info: The System.Runtime.Serialization.SerializationInfo object to populate with serialization 

    information.

  

   context: (Reserved) The place to store and retrieve serialized data.
  """
  pass
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==y """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __repr__(self,*args):
  """ __repr__(self: object) -> str """
  pass
 def __str__(self,*args):
  pass
 Value=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a handle to the field represented by the current instance.



Get: Value(self: RuntimeFieldHandle) -> IntPtr



"""





class PlatformID(Enum,IComparable,IFormattable,IConvertible):
 """
 Identifies the operating system,or platform,supported by an assembly.

 

 enum PlatformID,values: MacOSX (6),Unix (4),Win32NT (2),Win32S (0),Win32Windows (1),WinCE (3),Xbox (5)
 """
 def __eq__(self,*args):
  """ x.__eq__(y) <==> x==yx.__eq__(y) <==> x==yx.__eq__(y) <==> x==y """
  pass
 def __format__(self,*args):
  """ __format__(formattable: IFormattable,format: str) -> str """
  pass
 def __ge__(self,*args):
  pass
 def __gt__(self,*args):
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __le__(self,*args):
  pass
 def __lt__(self,*args):
  pass
 def __ne__(self,*args):
  pass
 def __reduce_ex__(self,*args):
  pass
 def __str__(self,*args):
  pass
 MacOSX=None
 Unix=None
 value__=None
 Win32NT=None
 Win32S=None
 Win32Windows=None
 WinCE=None
 Xbox=None




class IBindingListView(IBindingList,IList,ICollection,IEnumerable):
 """ Extends the System.ComponentModel.IBindingList interface by providing advanced sorting and filtering capabilities. """
 def ApplySort(self,sorts):
  """
  ApplySort(self: IBindingListView,sorts: ListSortDescriptionCollection)

   Sorts the data source based on the given System.ComponentModel.ListSortDescriptionCollection.

  

   sorts: The System.ComponentModel.ListSortDescriptionCollection containing the sorts to apply to the 

    data source.
  """
  pass
 def RemoveFilter(self):
  """
  RemoveFilter(self: IBindingListView)

   Removes the current filter applied to the data source.
  """
  pass
 def __contains__(self,*args):
  """
  __contains__(self: IList,value: object) -> bool

  

   Determines whether the System.Collections.IList contains a specific value.

  

   value: The object to locate in the System.Collections.IList.

   Returns: true if the System.Object is found in the System.Collections.IList; otherwise,false.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 def __iter__(self,*args):
  """ __iter__(self: IEnumerable) -> object """
  pass
 def __len__(self,*args):
  """ x.__len__() <==> len(x) """
  pass
 Filter=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets the filter to be used to exclude items from the collection of items returned by the data source



Get: Filter(self: IBindingListView) -> str



Set: Filter(self: IBindingListView)=value

"""

 SortDescriptions=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the collection of sort descriptions currently applied to the data source.



Get: SortDescriptions(self: IBindingListView) -> ListSortDescriptionCollection



"""

 SupportsAdvancedSorting=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the data source supports advanced sorting.



Get: SupportsAdvancedSorting(self: IBindingListView) -> bool



"""

 SupportsFiltering=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a value indicating whether the data source supports filtering.



Get: SupportsFiltering(self: IBindingListView) -> bool



"""





class HtmlElementErrorEventArgs(EventArgs):
 """ Provides data for the System.Windows.Forms.HtmlWindow.Error event. """
 Description=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the descriptive string corresponding to the error.



Get: Description(self: HtmlElementErrorEventArgs) -> str



"""

 Handled=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets whether this error has been handled by the application hosting the document.



Get: Handled(self: HtmlElementErrorEventArgs) -> bool



Set: Handled(self: HtmlElementErrorEventArgs)=value

"""

 LineNumber=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the line of HTML script code on which the error occurred.



Get: LineNumber(self: HtmlElementErrorEventArgs) -> int



"""

 Url=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the location of the document that generated the error.



Get: Url(self: HtmlElementErrorEventArgs) -> Uri



"""





class SafeBuffer(SafeHandleZeroOrMinusOneIsInvalid,IDisposable):
 """ Provides a controlled memory buffer that can be used for reading and writing. Attempts to access memory outside the controlled buffer (underruns and overruns) raise exceptions. """
 def AcquirePointer(self,pointer):
  """
  AcquirePointer(self: SafeBuffer,pointer: Byte*) -> Byte*

  

   Obtains a pointer from a System.Runtime.InteropServices.SafeBuffer object for a block of memory.

  

   pointer: A byte pointer,passed by reference,to receive the pointer from within the 

    System.Runtime.InteropServices.SafeBuffer object. You must set this pointer to null before you 

    call this method.
  """
  pass
 def Dispose(self):
  """
  Dispose(self: SafeHandle,disposing: bool)

   Releases the unmanaged resources used by the System.Runtime.InteropServices.SafeHandle class 

    specifying whether to perform a normal dispose operation.

  

  

   disposing: true for a normal dispose operation; false to finalize the handle.
  """
  pass
 def Initialize(self,*__args):
  """
  Initialize[T](self: SafeBuffer,numElements: UInt32)Initialize(self: SafeBuffer,numElements: UInt32,sizeOfEachElement: UInt32)

   Specifies the allocation size of the memory buffer by using the specified number of elements and 

    element size. You must call this method before you use the 

    System.Runtime.InteropServices.SafeBuffer instance.

  

  

   numElements: The number of elements in the buffer.

   sizeOfEachElement: The size of each element in the buffer.

  Initialize(self: SafeBuffer,numBytes: UInt64)

   Defines the allocation size of the memory region in bytes. You must call this method before you 

    use the System.Runtime.InteropServices.SafeBuffer instance.

  

  

   numBytes: The number of bytes in the buffer.
  """
  pass
 def Read(self,byteOffset):
  """ Read[T](self: SafeBuffer,byteOffset: UInt64) -> T """
  pass
 def ReadArray(self,byteOffset,array,index,count):
  """ ReadArray[T](self: SafeBuffer,byteOffset: UInt64,array: Array[T],index: int,count: int) """
  pass
 def ReleaseHandle(self,*args):
  """
  ReleaseHandle(self: SafeHandle) -> bool

  

   When overridden in a derived class,executes the code required to free the handle.

   Returns: true if the handle is released successfully; otherwise,in the event of a catastrophic failure,

    false. In this case,it generates a releaseHandleFailed MDA Managed Debugging Assistant.
  """
  pass
 def ReleasePointer(self):
  """
  ReleasePointer(self: SafeBuffer)

   Releases a pointer that was obtained by the 

    System.Runtime.InteropServices.SafeBuffer.AcquirePointer(System.Byte*@) method.
  """
  pass
 def SetHandle(self,*args):
  """
  SetHandle(self: SafeHandle,handle: IntPtr)

   Sets the handle to the specified pre-existing handle.

  

   handle: The pre-existing handle to use.
  """
  pass
 def Write(self,byteOffset,value):
  """ Write[T](self: SafeBuffer,byteOffset: UInt64,value: T) """
  pass
 def WriteArray(self,byteOffset,array,index,count):
  """ WriteArray[T](self: SafeBuffer,byteOffset: UInt64,array: Array[T],index: int,count: int) """
  pass
 def __enter__(self,*args):
  """
  __enter__(self: IDisposable) -> object

  

   Provides the implementation of __enter__ for objects which implement IDisposable.
  """
  pass
 def __exit__(self,*args):
  """
  __exit__(self: IDisposable,exc_type: object,exc_value: object,exc_back: object)

   Provides the implementation of __exit__ for objects which implement IDisposable.
  """
  pass
 def __init__(self,*args):
  """ x.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signaturex.__init__(...) initializes x; see x.__class__.__doc__ for signature """
  pass
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """ __new__(cls: type,ownsHandle: bool) """
  pass
 ByteLength=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets the size of the buffer,in bytes.



Get: ByteLength(self: SafeBuffer) -> UInt64



"""


 handle=None




class NativeObjectSecurity(CommonObjectSecurity):
 """ Provides the ability to control access to native objects without direct manipulation of Access Control Lists (ACLs). Native object types are defined by the System.Security.AccessControl.ResourceType enumeration. """
 @staticmethod
 def __new__(self,*args): #cannot find CLR constructor
  """
  __new__(cls: type,isContainer: bool,resourceType: ResourceType)

  __new__(cls: type,isContainer: bool,resourceType: ResourceType,exceptionFromErrorCode: ExceptionFromErrorCode,exceptionContext: object)

  __new__(cls: type,isContainer: bool,resourceType: ResourceType,name: str,includeSections: AccessControlSections,exceptionFromErrorCode: ExceptionFromErrorCode,exceptionContext: object)

  __new__(cls: type,isContainer: bool,resourceType: ResourceType,name: str,includeSections: AccessControlSections)

  __new__(cls: type,isContainer: bool,resourceType: ResourceType,handle: SafeHandle,includeSections: AccessControlSections,exceptionFromErrorCode: ExceptionFromErrorCode,exceptionContext: object)

  __new__(cls: type,isContainer: bool,resourceType: ResourceType,handle: SafeHandle,includeSections: AccessControlSections)
  """
  pass
 AccessRulesModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the access rules associated with this System.Security.AccessControl.ObjectSecurity object have been modified.



"""

 AuditRulesModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the audit rules associated with this System.Security.AccessControl.ObjectSecurity object have been modified.



"""

 GroupModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the group associated with the securable object has been modified.



"""

 IsContainer=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a Boolean value that specifies whether this System.Security.AccessControl.ObjectSecurity object is a container object.



"""

 IsDS=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets a Boolean value that specifies whether this System.Security.AccessControl.ObjectSecurity object is a directory object.



"""

 OwnerModified=property(lambda self: object(),lambda self,v: None,lambda self: None)
 """Gets or sets a Boolean value that specifies whether the owner of the securable object has been modified.



"""


 ExceptionFromErrorCode=None




"""Utility functions to support forms module."""

from pyrevit import framework
from pyrevit.framework import wpf, Controls, Imaging, Uri, UriKind


def bitmap_from_file(bitmap_file):
    """Create BitmapImage from a bitmap file.

    Args:
        bitmap_file (str): absolute path to bitmap file

    Returns:
        (BitmapImage): bitmap image object
    """
    bitmap = Imaging.BitmapImage()
    bitmap.BeginInit()
    success, uri = Uri.TryCreate(bitmap_file, UriKind.RelativeOrAbsolute)
    if not success or uri is None:
        raise ValueError("Failed to create Uri from file path: {}".format(bitmap_file))
    bitmap.UriSource = uri
    bitmap.CacheOption = Imaging.BitmapCacheOption.OnLoad
    bitmap.CreateOptions = Imaging.BitmapCreateOptions.IgnoreImageCache
    bitmap.EndInit()
    bitmap.Freeze()
    return bitmap


def load_component(xaml_file, comp_type):
    """Load WPF component from xaml file.

    Args:
        xaml_file (str): xaml file path
        comp_type (System.Windows.Controls): WPF control type

    Returns:
        (System.Windows.Controls): loaded WPF control
    """
    return wpf.LoadComponent(comp_type, xaml_file)


def load_ctrl_template(xaml_file):
    """Load System.Windows.Controls.ControlTemplate from xaml file.

    Args:
        xaml_file (str): xaml file path

    Returns:
        (System.Windows.Controls.ControlTemplate): loaded control template
    """
    return load_component(xaml_file, Controls.ControlTemplate())


def load_itemspanel_template(xaml_file):
    """Load System.Windows.Controls.ItemsPanelTemplate from xaml file.

    Args:
        xaml_file (str): xaml file path

    Returns:
        (System.Windows.Controls.ControlTemplate): loaded items-panel template
    """
    return load_component(xaml_file, Controls.ItemsPanelTemplate())
